----------------------------------------------------------------------
-- timer_cnt
----------------------------------------------------------------------
-- Revision 00
-- Level	00
----------------------------------------------------------------------
-- File			: timer_cnt.vhd
-- Entity		: timer_cnt
-- Architecture	: RTL
-- Package		: gp
-- History -----------------------------------------------------------
-- Rev.Level	Date		Cause		Coded					Contents
-- 00.00		08/01/2011									new
----------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity timer_cnt is
    generic (
		CLK_SPEED_MHZ			: INTEGER := 400						-- CLKXs[h(400MHz)
	);
  port (
		-- RESET / CLK
		RST						: in	std_logic;						-- 1:Zbg
		CLK						: in	std_logic;						-- 1:Zbg
		-- ^C}o̓pX
		timer_1us_p				: out	std_logic;						-- 1us^C}o̓pX
		timer_1ms_p				: out	std_logic;						-- 1ms^C}o̓pX
		timer_1s_p				: out	std_logic						-- 1s ^C}o̓pX
	);
end timer_cnt;

architecture rtl of timer_cnt is

--------------------------------------------------
-- Components 
--------------------------------------------------

--------------------------------------------------
-- signals
--------------------------------------------------
signal	tim_1us_cnt				: std_logic_vector(8 downto 0);			-- CLK`FbN
signal	tim_1us_cnt_p			: std_logic;							-- CLK`FbN
signal	tim_1ms_cnt				: std_logic_vector(9 downto 0);			-- CLK`FbN
signal	tim_1ms_cnt_p			: std_logic;							-- CLK`FbN
signal	tim_1s_cnt				: std_logic_vector(9 downto 0);			-- CLK`FbN
signal	tim_1s_cnt_p			: std_logic;							-- CLK`FbN
signal	DEBUG_FLG				: std_logic;

begin

	--------------------------------------------------------------------
	-- DEBUG_FLG
	--------------------------------------------------------------------
	DEBUG_FLG	<= '0';

	--------------------------------------------------------------------
	-- ^C}o̓pX
	--------------------------------------------------------------------
	process(CLK,RST) begin
		if (RST = '1') then
			timer_1us_p		<= '0';										-- 1us^C}o̓pX
			timer_1ms_p		<= '0';										-- 1ms^C}o̓pX
			timer_1s_p		<= '0';										-- 1s ^C}o̓pX
		elsif (CLK'event and CLK='1') then 
			if(DEBUG_FLG = '0')then
				timer_1us_p		<= tim_1us_cnt_p;
				timer_1ms_p		<= tim_1ms_cnt_p;
				timer_1s_p		<= tim_1s_cnt_p;
			else
				timer_1us_p		<= tim_1us_cnt_p;
				timer_1ms_p		<= tim_1us_cnt_p;
				timer_1s_p		<= tim_1us_cnt_p;
			end if;
		end if;
	end process;
	--------------------------------------------------------------------
	-- ^C}
	--------------------------------------------------------------------
	-- 1 us JE^
	process(CLK,RST) begin
		if (RST = '1') then
			tim_1us_cnt		<= (others => '0');							-- CLK`FbN
			tim_1us_cnt_p	<= '0';										-- CLK`FbN
		elsif (CLK'event and CLK='1') then 
			if(tim_1us_cnt = CLK_SPEED_MHZ - 1)then
				tim_1us_cnt		<= (others => '0');
				tim_1us_cnt_p	<= '1';
			else
				tim_1us_cnt		<= tim_1us_cnt + 1;
				tim_1us_cnt_p	<= '0';
			end if;
		end if;
	end process;
	-- 1 ms JE^
	process(CLK,RST) begin
		if (RST = '1') then
			tim_1ms_cnt		<= (others => '0');							-- CLK`FbN
			tim_1ms_cnt_p	<= '0';										-- CLK`FbN
		elsif (CLK'event and CLK='1') then 
			if(tim_1us_cnt_p = '1')then
				if(tim_1ms_cnt = 1000 - 1)then
					tim_1ms_cnt		<= (others => '0');
					tim_1ms_cnt_p	<= '1';	
				else
					tim_1ms_cnt		<= tim_1ms_cnt + 1;
				end if;
			else
				tim_1ms_cnt_p	<= '0';	
			end if;
		end if;
	end process;
	-- 1 s JE^
	process(CLK,RST) begin
		if (RST = '1') then
			tim_1s_cnt		<= (others => '0');							-- CLK`FbN
			tim_1s_cnt_p	<= '0';										-- CLK`FbN
		elsif (CLK'event and CLK='1') then 
			if(tim_1ms_cnt_p = '1')then
				if(tim_1s_cnt = 1000 - 1)then
					tim_1s_cnt		<= (others => '0');
					tim_1s_cnt_p	<= '1';	
				else
					tim_1s_cnt		<= tim_1s_cnt + 1;
				end if;
			else
				tim_1s_cnt_p	<= '0';	
			end if;
		end if;
	end process;

end rtl;
