----------------------------------------------------------------------
-- SKY_PCIE_KC705
----------------------------------------------------------------------
-- Revision 00
-- Level	00
---------------------------------------------------------------------- 
-- File			: SKY_PCIE_KC705.vhd
-- Entity		: SKY_PCIE_KC705
-- Architecture	: RTL
-- Package		: gp
-- History -----------------------------------------------------------
-- Rev.Level	Date		Cause		Coded					Contents
-- 00.00		10/31/2011			SKYWAVE					new
----------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all; 
library unisim;
use unisim.vcomponents.all;

entity SKY_PCIE_KC705 is
	generic (
		MEM_AREA				: std_logic_vector(31 downto 0) := X"40000000";	-- 1GBYTE
		ABITS_MIG_RD			: INTEGER := 8								;
		MIG_DT_BITS				: integer range 128 to 512 := 512			;
		MIG_DB_CYCLE			: integer range 1 to 2 := 1					;	-- app_xdf ]NbN
		TRN_DT_BITS				: INTEGER := 128							;
		LINK_CAP_MAX_LINK_WIDTH	: INTEGER := 8								;
		UMWT_TRN_DT_BITS		: INTEGER := 512;
		UMRD_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
		UMRD_TRN_DT_BITS		: INTEGER range 128 to 512 := 512		-- 512p
	);
    port(
--<< Zbg >>
		SYSCLK_P                : in	std_logic;  					--	200MHzNbN LOC = AD12 | IOSTANDARD=LVDS;     # Bank  33 VCCO - VCC1V5_FPGA - IO_L12P_T1_MRCC_33
		SYSCLK_N                : in	std_logic;  					--	200MHzNbN LOC = AD11 | IOSTANDARD=LVDS;     # Bank  33 VCCO - VCC1V5_FPGA - IO_L12N_T1_MRCC_33
		CPU_RESET    			: in	std_logic; 						-- 1:PUSH LOC = AB7  | IOSTANDARD=LVCMOS15; # Bank  34 VCCO - VCC1V5_FPGA - IO_25_VRP_34
--<< USER_SMA_GPIO_P >>
		USER_SMA_GPIO_P			: out 	std_logic := '0';
		USER_SMA_GPIO_N			: out 	std_logic := '0';
--<< PCIE >>
		pci_exp_txp             : out 	std_logic_vector(LINK_CAP_MAX_LINK_WIDTH-1 downto 0);
  		pci_exp_txn             : out 	std_logic_vector(LINK_CAP_MAX_LINK_WIDTH-1 downto 0);
  		pci_exp_rxp             : in 	std_logic_vector(LINK_CAP_MAX_LINK_WIDTH-1 downto 0);
  		pci_exp_rxn             : in 	std_logic_vector(LINK_CAP_MAX_LINK_WIDTH-1 downto 0);
		sys_clk_p	      		: in 	std_logic;
		sys_clk_n   	        : in 	std_logic;
		sys_rst_n   			: in 	std_logic;

--<< LED >>
		GPIO_LED_0_LS           : out 	std_logic := '0';				--	LOC = AB8  | IOSTANDARD=LVCMOS15; # Bank  33 VCCO - VCC1V5_FPGA - IO_L2N_T0_33
		GPIO_LED_1_LS           : out 	std_logic := '0';				--	LOC = AA8  | IOSTANDARD=LVCMOS15; # Bank  33 VCCO - VCC1V5_FPGA - IO_L2P_T0_33
		GPIO_LED_2_LS           : out 	std_logic := '0';				--	LOC = AC9  | IOSTANDARD=LVCMOS15; # Bank  33 VCCO - VCC1V5_FPGA - IO_L3N_T0_DQS_33
		GPIO_LED_3_LS           : out 	std_logic := '0';				--	LOC = AB9  | IOSTANDARD=LVCMOS15; # Bank  33 VCCO - VCC1V5_FPGA - IO_L3P_T0_DQS_33
		GPIO_LED_4_LS           : out 	std_logic := '0';				--	LOC = AE26 | IOSTANDARD=LVCMOS18; # Bank  13 VCCO - VADJ_FPGA - IO_25_13
		GPIO_LED_5_LS           : out 	std_logic := '0';				--	LOC = G19  | IOSTANDARD=LVCMOS18; # Bank  17 VCCO - VADJ_FPGA - IO_0_17
		GPIO_LED_6_LS           : out 	std_logic := '0';				--	LOC = E18  | IOSTANDARD=LVCMOS18; # Bank  17 VCCO - VADJ_FPGA - IO_25_17
		GPIO_LED_7_LS           : out 	std_logic := '0';				--	LOC = F16  | IOSTANDARD=LVCMOS18; # Bank  18 VCCO - VADJ_FPGA - IO_25_18
--<< SW >>
		GPIO_DIP_SW0            : in	std_logic;						--	LOC = Y29  | IOSTANDARD=LVCMOS18; # Bank  13 VCCO - VADJ_FPGA - IO_L4N_T0_13
		GPIO_DIP_SW1            : in	std_logic;						--	LOC = W29  | IOSTANDARD=LVCMOS18; # Bank  13 VCCO - VADJ_FPGA - IO_L4P_T0_13
		GPIO_DIP_SW2            : in	std_logic;						--	LOC = AA28 | IOSTANDARD=LVCMOS18; # Bank  13 VCCO - VADJ_FPGA - IO_L3N_T0_DQS_13
		GPIO_DIP_SW3            : in	std_logic;						--	LOC = Y28  | IOSTANDARD=LVCMOS18; # Bank  13 VCCO - VADJ_FPGA - IO_L3P_T0_DQS_13
		GPIO_SW_C               : in	std_logic;						--	LOC = G12  | IOSTANDARD=LVCMOS18; # Bank  18 VCCO - VADJ_FPGA - IO_0_18
		GPIO_SW_N               : in	std_logic;						--	LOC = AA12 | IOSTANDARD=LVCMOS15; # Bank  33 VCCO - VCC1V5_FPGA - IO_L1P_T0_33
		GPIO_SW_S               : in	std_logic;						--	LOC = AB12 | IOSTANDARD=LVCMOS15; # Bank  33 VCCO - VCC1V5_FPGA - IO_L1N_T0_33
		GPIO_SW_W               : in	std_logic;						--	LOC = AC6  | IOSTANDARD=LVCMOS15; # Bank  34 VCCO - VCC1V5_FPGA - IO_0_VRN_34
		GPIO_SW_E               : in	std_logic;						--	LOC = AG5  | IOSTANDARD=LVCMOS15; # Bank  34 VCCO - VCC1V5_FPGA - IO_L12N_T1_MRCC_34
--<< FAN >>
		SM_FAN_PWM              : out 	std_logic;						--	LOC = L26  | IOSTANDARD=LVCMOS25; # Bank  15 VCCO - VCC2V5_FPGA - IO_L11P_T1_SRCC_AD12P_15
-- DIMM
		ddr3_dq       			: inout std_logic_vector(63 downto 0);
		ddr3_dm       			: out   std_logic_vector(7 downto 0);
		ddr3_addr     			: out   std_logic_vector(13 downto 0);
		ddr3_ba       			: out   std_logic_vector(2 downto 0);
		ddr3_ras_n    			: out   std_logic;
		ddr3_cas_n    			: out   std_logic;
		ddr3_we_n     			: out   std_logic;
		ddr3_reset_n  			: out   std_logic;
		ddr3_cs_n     			: out   std_logic_vector(0 downto 0);
		ddr3_odt      			: out   std_logic_vector(0 downto 0);
		ddr3_cke      			: out   std_logic_vector(0 downto 0);
		ddr3_dqs_p    			: inout std_logic_vector(7 downto 0);
		ddr3_dqs_n    			: inout std_logic_vector(7 downto 0);
		ddr3_ck_p     			: out   std_logic_vector(0 downto 0);
		ddr3_ck_n     			: out   std_logic_vector(0 downto 0)
    );
end SKY_PCIE_KC705;

--------------------------------------------------
-- Architecture Body Define
--------------------------------------------------
architecture RTL of SKY_PCIE_KC705 is

--------------------------------------------------
-- Components
--------------------------------------------------
component xilinx_pcie_2_1_ep_7x
  generic (
  PCIE_EXT_CLK                  : string :="TRUE";  -- Use External Clocking Module
  PL_FAST_TRAIN                 : string := "FALSE";
  C_DATA_WIDTH                  : integer range 64 to 128 := 128
  );
  port (
  pci_exp_txp                   : out std_logic_vector(7 downto 0);
  pci_exp_txn                   : out std_logic_vector(7 downto 0);
  pci_exp_rxp                   : in std_logic_vector(7 downto 0);
  pci_exp_rxn                   : in std_logic_vector(7 downto 0);

  sys_clk_p                     : in std_logic;
  sys_clk_n                     : in std_logic;
  sys_rst_n                     : in std_logic;


--  component pcie_app_7x
		user_clk_out                   : out std_logic;													
		user_reset_out                 : out std_logic;                                                 
		user_lnk_up_out                : out std_logic;                                                 
		s_axis_tx_tdata                : in std_logic_vector((C_DATA_WIDTH - 1) downto 0);              
		s_axis_tx_tkeep                : in std_logic_vector((C_DATA_WIDTH / 8 - 1) downto 0);          
		s_axis_tx_tlast                : in std_logic;                                                  
		s_axis_tx_tvalid               : in std_logic;                                                  
		s_axis_tx_tready               : out std_logic;                                                 
		s_axis_tx_tuser                : in std_logic_vector(3 downto 0);                               
		tx_cfg_req                     : out std_logic;                                                 
		tx_cfg_gnt                     : in std_logic;                                                  
		tx_err_drop                    : out std_logic;                                                 
		tx_buf_av                      : out std_logic_vector(5 downto 0);                              
		m_axis_rx_tdata                : out std_logic_vector((C_DATA_WIDTH - 1) downto 0);             
		m_axis_rx_tkeep                : out std_logic_vector((C_DATA_WIDTH / 8 - 1) downto 0);         
		m_axis_rx_tlast                : out std_logic;                                                 
		m_axis_rx_tvalid               : out std_logic;                                                 
		m_axis_rx_tready               : in std_logic;                                                  
		m_axis_rx_tuser                : out std_logic_vector(21 downto 0);                             
		rx_np_ok                       : in std_logic;                                                  
		rx_np_req                      : in std_logic;                                                  
		fc_nph                         : out std_logic_vector(7 downto 0);                              
		fc_npd                         : out std_logic_vector(11 downto 0);                             
		fc_ph                          : out std_logic_vector(7 downto 0);                              
		fc_pd                          : out std_logic_vector(11 downto 0);                             
		fc_cplh                        : out std_logic_vector(7 downto 0);                              
		fc_cpld                        : out std_logic_vector(11 downto 0);                             
		fc_sel                         : in std_logic_vector(2 downto 0);                               
		cfg_err_cor                    : in std_logic;                                                  
		cfg_err_ur                     : in std_logic;                                                  
		cfg_err_cpl_rdy                : out std_logic;                                                 
		cfg_err_ecrc                   : in std_logic;                                                  
		cfg_err_cpl_timeout            : in std_logic;                                                  
		cfg_err_cpl_abort              : in std_logic;                                                  
		cfg_err_cpl_unexpect           : in std_logic;                                                  
		cfg_err_posted                 : in std_logic;                                                  
		cfg_err_locked                 : in std_logic;                                                  
		cfg_err_atomic_egress_blocked  : in std_logic;                                                  
		cfg_err_internal_cor           : in std_logic;                                                  
		cfg_err_malformed              : in std_logic;                                                  
		cfg_err_mc_blocked             : in std_logic;                                                  
		cfg_err_poisoned               : in std_logic;                                                  
		cfg_err_norecovery             : in std_logic;                                                  
		cfg_err_acs                    : in std_logic;                                                  
		cfg_err_internal_uncor         : in std_logic;                                                  
		cfg_err_aer_headerlog          : in std_logic_vector(127 downto 0);                             
		cfg_aer_interrupt_msgnum       : in std_logic_vector(4 downto 0);                               
		cfg_err_aer_headerlog_set      : out  std_logic;                                                
		cfg_aer_ecrc_check_en          : out  std_logic;                                                
		cfg_aer_ecrc_gen_en            : out  std_logic;                                                
		cfg_interrupt                  : in std_logic;                                                  
		cfg_interrupt_rdy              : out std_logic;                                                 
		cfg_interrupt_assert           : in std_logic;                                                  
		cfg_interrupt_di               : in std_logic_vector(7 downto 0);                               
		cfg_interrupt_do               : out  std_logic_vector(7 downto 0);                             
		cfg_interrupt_mmenable         : out  std_logic_vector(2 downto 0);                             
		cfg_interrupt_msienable        : out  std_logic;                                                
		cfg_interrupt_msixenable       : out  std_logic;                                                
		cfg_interrupt_msixfm           : out  std_logic;                                                
		cfg_interrupt_stat             : in std_logic;                                                  
		cfg_pciecap_interrupt_msgnum   : in std_logic_vector( 4 downto 0);                              
		cfg_turnoff_ok                 : in std_logic;                                                  
		cfg_to_turnoff                 : out std_logic;                                                 
		cfg_pm_wake                    : in std_logic;                                                  
		cfg_pm_force_state_en          : in std_logic;                                                  
		cfg_pm_force_state             : in std_logic_vector( 1 downto 0);                              
		cfg_pm_halt_aspm_l0s           : in std_logic;                                                  
		cfg_pm_halt_aspm_l1            : in std_logic;                                                  
		cfg_pcie_link_state            : out std_logic_vector(2 downto 0);                              
		cfg_trn_pending                : in std_logic;                                                  
		cfg_err_tlp_cpl_header         : in std_logic_vector(47 downto 0);                              
		cfg_bus_number                 : out std_logic_vector(7 downto 0);                              
		cfg_device_number              : out std_logic_vector(4 downto 0);                              
		cfg_function_number            : out std_logic_vector(2 downto 0);                              
		cfg_status                     : out std_logic_vector(15 downto 0);                             
		cfg_command                    : out std_logic_vector(15 downto 0);                             
		cfg_dstatus                    : out std_logic_vector(15 downto 0);                             
		cfg_dcommand                   : out std_logic_vector(15 downto 0);                             
		cfg_lstatus                    : out std_logic_vector(15 downto 0);                             
		cfg_lcommand                   : out std_logic_vector(15 downto 0);                             
		cfg_dcommand2                  : out std_logic_vector(15 downto 0);                             
		cfg_mgmt_di                    : in std_logic_vector(31 downto 0);                              
		cfg_mgmt_byte_en               : in std_logic_vector( 3 downto 0);                              
		cfg_mgmt_dwaddr                : in std_logic_vector( 9 downto 0);                              
		cfg_mgmt_wr_en                 : in std_logic;                                                  
		cfg_mgmt_rd_en                 : in std_logic;                                                  
		cfg_mgmt_wr_readonly           : in std_logic;                                                  
		pl_directed_link_change        : in std_logic_vector(1 downto 0);                               
		pl_ltssm_state                 : out std_logic_vector(5 downto 0);                              
		pl_directed_link_width         : in std_logic_vector(1 downto 0);                               
		pl_directed_link_speed         : in std_logic;                                                  
		pl_directed_link_auton         : in std_logic;                                                  
		pl_upstream_prefer_deemph      : in std_logic;                                                  
		pl_sel_lnk_width               : out std_logic_vector(1 downto 0);                              
		pl_sel_lnk_rate                : out std_logic;                                                 
		pl_link_gen2_cap               : out std_logic;                                                 
		pl_link_partner_gen2_supported : out std_logic;                                                 
		pl_initial_link_width          : out std_logic_vector(2 downto 0);                              
		pl_link_upcfg_cap              : out std_logic;                                                 
		pl_lane_reversal_mode          : out std_logic_vector(1 downto 0);                              
		pl_received_hot_rst            : out std_logic;                                                 
		cfg_dsn                        : in std_logic_vector(63 downto 0)                               

);
end component;
component iSKY_PCIe
	generic (
--		FAST_SIMULATION			: INTEGER := 0;
--		PCIE4_MAX_PAYLOAD		: std_logic_vector(9 downto 0) := "0100000000";	-- 256BYTE
		TRN_DT_BITS				: INTEGER := 64;
		MIG_DT_BITS				: integer range 128 to 512 := 256;
		MIG_DB_CYCLE			: integer range 1 to 2 := 1;			-- app_xdf ]NbN
--
--		MWT_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
--		MWT_AFLEV				: INTEGER := 4;							-- 2**AFLEV=̂16AhXafull=1ƂȂ
----		MWT_TRN_DT_BITS			: INTEGER := 256;
--
--		MRD_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
--		MRD_AFLEV				: INTEGER := 4;							-- 2**AFLEV=̂16AhXafull=1ƂȂ
----		MRD_TRN_DT_BITS			: INTEGER := 64;						-- 
--
--		UMWT_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
--		UMWT_AFLEV				: INTEGER := 4;							-- 2**AFLEV=̂16AhXafull=1ƂȂ
		UMWT_TRN_DT_BITS		: INTEGER := 64;
		UMWT_TRN_MK_BITS		: INTEGER := 2;
--
		UMRD_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
--		UMRD_AFLEV				: INTEGER := 4;							-- 2**AFLEV=̂16AhXafull=1ƂȂ
		UMRD_TRN_DT_BITS		: INTEGER := 64
--
--		MR_FLG_ABITS_AD			: INTEGER := 9							-- UMRD_ABITS_AD,MRD_ABITS_AD̃f[^eʂw
	);
	port (
--<< PCI Express >>
		user_clk                       	: in  	std_logic;
		user_reset                     	: in  	std_logic;
		user_lnk_up                    	: in  	std_logic;
		
		-- Tx
		tx_buf_av                      	: in  	std_logic_vector(5 downto 0);
		tx_cfg_req                     	: in  	std_logic;
		tx_err_drop                    	: in  	std_logic;
		tx_cfg_gnt                     	: out 	std_logic;
		
		s_axis_tx_tready               	: in  	std_logic;
		s_axis_tx_tdata                	: out 	std_logic_vector((TRN_DT_BITS - 1) downto 0);
		s_axis_tx_tkeep                	: out 	std_logic_vector((TRN_DT_BITS/8)-1 downto 0);
		s_axis_tx_tuser                	: out 	std_logic_vector(3 downto 0);
		s_axis_tx_tlast                	: out 	std_logic;
		s_axis_tx_tvalid               	: out 	std_logic;
		
		-- RX
		rx_np_ok                       	: out 	std_logic;
		rx_np_req                      	: out 	std_logic;
		m_axis_rx_tdata                	: in 	std_logic_vector((TRN_DT_BITS - 1) downto 0);
		m_axis_rx_tkeep                	: in 	std_logic_vector((TRN_DT_BITS/8)-1 downto 0);
		m_axis_rx_tlast                	: in  	std_logic;
		m_axis_rx_tvalid               	: in  	std_logic;
		m_axis_rx_tready               	: out 	std_logic;
		m_axis_rx_tuser                	: in 	std_logic_vector(21 downto 0);
		
		-- Flow Control
		fc_cpld                        	: in  	std_logic_vector(11 downto 0);
		fc_cplh                        	: in  	std_logic_vector(7 downto 0);
		fc_npd                         	: in  	std_logic_vector(11 downto 0);
		fc_nph                         	: in  	std_logic_vector(7 downto 0);
		fc_pd                          	: in  	std_logic_vector(11 downto 0);
		fc_ph                          	: in  	std_logic_vector(7 downto 0);
		fc_sel                         	: out 	std_logic_vector(2 downto 0);
		
		-- CFG
		
		cfg_err_cor                    	: out 	std_logic;
		cfg_err_ur                     	: out 	std_logic;
		cfg_err_ecrc                   	: out 	std_logic;
		cfg_err_cpl_timeout            	: out 	std_logic;
		cfg_err_cpl_unexpect           	: out 	std_logic;
		cfg_err_cpl_abort              	: out 	std_logic;
		cfg_err_atomic_egress_blocked  	: out 	std_logic;
		cfg_err_internal_cor           	: out 	std_logic;
		cfg_err_malformed              	: out 	std_logic;
		cfg_err_mc_blocked             	: out 	std_logic;
		cfg_err_poisoned               	: out 	std_logic;
		cfg_err_norecovery             	: out 	std_logic;
		cfg_err_acs                    	: out 	std_logic;
		cfg_err_internal_uncor         	: out 	std_logic;
		cfg_pm_halt_aspm_l0s           	: out 	std_logic;
		cfg_pm_halt_aspm_l1            	: out 	std_logic;
		cfg_pm_force_state_en          	: out 	std_logic;
		cfg_pm_force_state             	: out 	std_logic_vector( 1 downto 0);
		cfg_err_posted                 	: out 	std_logic;
		cfg_err_locked                 	: out 	std_logic;
		cfg_err_tlp_cpl_header         	: out 	std_logic_vector(47 downto 0);
		cfg_err_cpl_rdy                	: in  	std_logic;
		cfg_interrupt                  	: out 	std_logic;
		cfg_interrupt_rdy              	: in  	std_logic;
		cfg_interrupt_assert           	: out 	std_logic;
		cfg_interrupt_di               	: out 	std_logic_vector( 7 downto 0);
		cfg_interrupt_do               	: in  	std_logic_vector( 7 downto 0);
		cfg_interrupt_mmenable         	: in  	std_logic_vector( 2 downto 0);
		cfg_interrupt_msienable        	: in  	std_logic;
		cfg_interrupt_msixenable       	: in  	std_logic;
		cfg_interrupt_msixfm           	: in  	std_logic;
		cfg_turnoff_ok                 	: out 	std_logic;
		cfg_to_turnoff                 	: in  	std_logic;
		cfg_trn_pending                	: out 	std_logic;
		cfg_pm_wake                    	: out 	std_logic;
		cfg_bus_number                 	: in  	std_logic_vector( 7 downto 0);
		cfg_device_number              	: in  	std_logic_vector( 4 downto 0);
		cfg_function_number            	: in  	std_logic_vector( 2 downto 0);
		cfg_status                     	: in  	std_logic_vector(15 downto 0);
		cfg_command                    	: in  	std_logic_vector(15 downto 0);
		cfg_dstatus                    	: in  	std_logic_vector(15 downto 0);
		cfg_dcommand                   	: in  	std_logic_vector(15 downto 0);
		cfg_lstatus                    	: in  	std_logic_vector(15 downto 0);
		cfg_lcommand                   	: in  	std_logic_vector(15 downto 0);
		cfg_dcommand2                  	: in  	std_logic_vector(15 downto 0);
		cfg_pcie_link_state            	: in  	std_logic_vector( 2 downto 0);
		
		cfg_interrupt_stat             	: out 	std_logic;
		cfg_pciecap_interrupt_msgnum   	: out 	std_logic_vector( 4 downto 0);
		
		pl_directed_link_change        	: out 	std_logic_vector( 1 downto 0);
		pl_ltssm_state                 	: in  	std_logic_vector( 5 downto 0);
		pl_directed_link_width         	: out 	std_logic_vector( 1 downto 0);
		pl_directed_link_speed         	: out 	std_logic;
		pl_directed_link_auton         	: out 	std_logic;
		pl_upstream_prefer_deemph      	: out 	std_logic;
		pl_sel_lnk_width               	: in  	std_logic_vector( 1 downto 0);
		pl_sel_lnk_rate                	: in  	std_logic;
		pl_link_gen2_cap               	: in  	std_logic;
		pl_link_partner_gen2_supported 	: in  	std_logic;
		pl_initial_link_width          	: in  	std_logic_vector( 2 downto 0);
		pl_link_upcfg_cap              	: in  	std_logic;
		pl_lane_reversal_mode          	: in  	std_logic_vector( 1 downto 0);
		pl_received_hot_rst            	: in  	std_logic;
		
		cfg_err_aer_headerlog          	: out 	std_logic_vector(127 downto 0);
		cfg_aer_interrupt_msgnum       	: out 	std_logic_vector(4 downto 0);
		cfg_err_aer_headerlog_set      	: in  	std_logic;
		cfg_aer_ecrc_check_en          	: in  	std_logic;
		cfg_aer_ecrc_gen_en            	: in  	std_logic;
		
		cfg_mgmt_di                    	: out 	std_logic_vector(31 downto 0);
		cfg_mgmt_byte_en               	: out 	std_logic_vector( 3 downto 0);
		cfg_mgmt_dwaddr                	: out 	std_logic_vector( 9 downto 0);
		cfg_mgmt_wr_en                 	: out 	std_logic;
		cfg_mgmt_rd_en                 	: out 	std_logic;
		cfg_mgmt_wr_readonly           	: out 	std_logic;
		
		cfg_dsn                        	: out 	std_logic_vector(63 downto 0);
-- << MIG >>
		mig_clk          				: in  	std_logic;
		app_addr      					: out	std_logic_vector(31 downto 0);	-- 
		app_cmd       					: out	std_logic_vector(2 downto 0);	-- 
		app_en        					: out	std_logic;						-- 
		app_rdy       					: in	std_logic;						-- 
                                
		app_wdf_wren  					: out	std_logic;						-- 
		app_wdf_data  					: out	std_logic_vector(MIG_DT_BITS-1 downto 0);	-- 
		app_wdf_mask					: out	std_logic_vector(MIG_DT_BITS/8-1 downto 0);
		app_wdf_end   					: out	std_logic;						-- 
		app_wdf_rdy   					: in	std_logic;						-- 
                                
		app_rd_data   					: in	std_logic_vector(MIG_DT_BITS-1 downto 0);	-- 
		app_rd_data_end 				: in	std_logic;						-- 
		app_rd_data_valid				: in	std_logic;						-- 
                                
-- << USER IF - GPIO >>
		user_gpio_out_00				: out	std_logic_vector(31 downto 0);
		user_gpio_out_01				: out	std_logic_vector(31 downto 0);
		user_gpio_out_02				: out	std_logic_vector(31 downto 0);
		user_gpio_out_03				: out	std_logic_vector(31 downto 0);
		user_gpio_in_00					: in	std_logic_vector(31 downto 0);
		user_gpio_in_01					: in	std_logic_vector(31 downto 0);
		user_gpio_in_02					: in	std_logic_vector(31 downto 0);
		user_gpio_in_03					: in	std_logic_vector(31 downto 0);
-- << USER IF - FIFO(MIG)>>
		user_mwr_clk					: in	std_logic;
		user_mwr_ad						: in	std_logic_vector(31 downto 6);
		user_mwr_ad_en					: in	std_logic;
		user_mwr_ad_afull				: out	std_logic;
		user_mwr_dt						: in	std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
		user_mwr_dt_en					: in	std_logic;
		user_mwr_mask					: in	std_logic_vector(UMWT_TRN_MK_BITS-1 downto 0);
                                
		user_mrd_clk					: in	std_logic;
		user_mrd_ad						: in	std_logic_vector(31 downto 6);
		user_mrd_ad_en					: in	std_logic;
		user_mrd_ad_afull				: out	std_logic;
		user_mrd_dt_en					: in	std_logic;
		user_mrd_dt						: out 	std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
		user_mrd_dt_count				: out 	std_logic_vector(UMRD_ABITS_AD-1 downto 0);
-- << 䃌WX^ANZX >>
		plbc_regc_ad					: in	std_logic_vector(6 downto 0);	-- AhX(2rbg͖)
		plbc_regc_wd					: in	std_logic_vector(31 downto 0);	-- 䃉Cgf[^
		plbc_regc_wp					: in	std_logic;						-- 䃉CgpX
		plbc_regc_rd					: out	std_logic_vector(31 downto 0)	-- 䃊[hf[^
	);
end component;
component timer_cnt 
    generic (
		CLK_SPEED_MHZ			: INTEGER := 400						-- CLKXs[h(400MHz)
	);
  port (
		-- RESET / CLK
		RST						: in	std_logic;						-- 1:Zbg
		CLK						: in	std_logic;						-- 1:Zbg
		-- ^C}o̓pX
		timer_1us_p				: out	std_logic;						-- 1us^C}o̓pX
		timer_1ms_p				: out	std_logic;						-- 1ms^C}o̓pX
		timer_1s_p				: out	std_logic						-- 1s ^C}o̓pX
	);
end component;
component mig_7series_v1_5
	port (
		
		-- Inouts     
		ddr3_dq       			: inout std_logic_vector(64-1 downto 0);
		ddr3_dqs_p    			: inout std_logic_vector(8-1 downto 0);
		ddr3_dqs_n    			: inout std_logic_vector(8-1 downto 0);
		
		-- Outputs
		ddr3_addr     			: out   std_logic_vector(14-1 downto 0);
		ddr3_ba       			: out   std_logic_vector(3-1 downto 0);
		ddr3_ras_n    			: out   std_logic;
		ddr3_cas_n    			: out   std_logic;
		ddr3_we_n     			: out   std_logic;
		ddr3_reset_n  			: out   std_logic;
		ddr3_ck_p     			: out   std_logic_vector(1-1 downto 0);
		ddr3_ck_n     			: out   std_logic_vector(1-1 downto 0);
		ddr3_cke      			: out   std_logic_vector(1-1 downto 0);
		ddr3_cs_n     			: out   std_logic_vector(1-1 downto 0);
		ddr3_dm       			: out   std_logic_vector(8-1 downto 0);
		ddr3_odt      			: out   std_logic_vector(1-1 downto 0);
		
   -- Inputs
   -- Differential system clocks
   sys_clk_p                      : in    std_logic;
   sys_clk_n                      : in    std_logic;
   -- Single-ended iodelayctrl clk (reference clock)
--   clk_ref_i                                : in    std_logic;
		-- user interface signals
		app_addr      			: in    std_logic_vector(28-1 downto 0);
		app_cmd       			: in    std_logic_vector(2 downto 0);
		app_en        			: in    std_logic;
		app_wdf_data  			: in    std_logic_vector(512-1 downto 0);
		app_wdf_end   			: in    std_logic;
		app_wdf_mask  			: in    std_logic_vector(64-1 downto 0)  ;
		app_wdf_wren  			: in    std_logic;
		app_rd_data   			: out   std_logic_vector(512-1 downto 0);
		app_rd_data_end   		: out   std_logic;
		app_rd_data_valid 		: out   std_logic;
		app_rdy         		: out   std_logic;
		app_wdf_rdy     		: out   std_logic;
		app_sr_req      		: in    std_logic;
		app_sr_active   		: out   std_logic;
		app_ref_req     		: in    std_logic;
		app_ref_ack     		: out   std_logic;
		app_zq_req      		: in    std_logic;
		app_zq_ack      		: out   std_logic;
		ui_clk          		: out   std_logic;
		ui_clk_sync_rst 		: out   std_logic;
		
		
		
		init_calib_complete		: out std_logic;
		
		
   -- System reset
      sys_rst                     : in    std_logic;
   -- sys_clk_i BUFG OUT
	  mmcm_clk_out 				  : out std_logic
	);
end component;
component usr_fifo_wrt 
	generic (
		UMWT_TRN_DT_BITS		: INTEGER := 128;
		MEM_AREA				: std_logic_vector(31 downto 0) := X"20000000"	-- 512MBYTE
	);
	port (
--<< NbN / Zbg >>
		RST						: in	std_logic;						-- Zbg
		CLK						: in	std_logic;						-- NbN
--<< ADC >>
		ADC_DT_0				: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_1				: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_2				: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_3				: in	std_logic_vector(15 downto 0);	-- ADC DATA
--<< M >>
		user_gpio_out_00		: in	std_logic_vector(31 downto 0);
		user_gpio_in_00			: out	std_logic_vector(31 downto 0);
--<< FIFOM >>
		user_mwr_ad				: out	std_logic_vector(31 downto 6);
		user_mwr_ad_en			: out	std_logic;
		user_mwr_ad_afull		: in	std_logic;
		user_mwr_dt				: out	std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
		user_mwr_dt_en			: out	std_logic
	);
end component;
component usr_fifo_rd
	generic (
		UMRD_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
		UMRD_TRN_DT_BITS		: INTEGER range 128 to 512 := 512;		-- 512p
		MEM_AREA				: std_logic_vector(31 downto 0) := X"20000000"	-- 512MBYTE
	);
	port (
--<< NbN / Zbg >>
		RST						: in	std_logic;						-- Zbg
		CLK						: in	std_logic;						-- NbN
--<< f[^o >>
		DT_OUT_0				: out	std_logic_vector(15 downto 0);
		DT_OUT_1				: out	std_logic_vector(15 downto 0);
		DT_OUT_2				: out	std_logic_vector(15 downto 0);
		DT_OUT_3				: out	std_logic_vector(15 downto 0);
--<< M >>
		user_gpio_out			: in	std_logic_vector(31 downto 0);
		user_gpio_in			: out	std_logic_vector(31 downto 0);
--<< FIFOM >>
		user_mrd_ad				: out	std_logic_vector(31 downto 6);
		user_mrd_ad_en			: out	std_logic;
		user_mrd_ad_afull		: in	std_logic;
		user_mrd_dt_en			: out	std_logic;
		user_mrd_dt				: in 	std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
		user_mrd_dt_count		: in 	std_logic_vector(UMRD_ABITS_AD-1 downto 0)
	);
end component;

--------------------------------------------------
-- Signals
--------------------------------------------------
signal	RST,XRST				: std_logic;
signal	CLK200MHz				: std_logic;
signal	CLK200_nobuf			: std_logic;
signal	timer_1s_p_clk200		: std_logic;

signal	timer_1ms_p				: std_logic;							-- 1ms^C}o̓pX
signal	timer_1s_p				: std_logic;							-- 1s ^C}o̓pX
signal	RUN_LED_buf				: std_logic;
signal	timer_500ms 			: std_logic_vector(8 downto 0);

-- PCIE CORE 
signal	user_clk                       	: std_logic;
signal	user_reset                     	: std_logic;
signal	user_lnk_up                    	: std_logic;
		
		-- Tx
signal	tx_buf_av                      	: std_logic_vector(5 downto 0);
signal	tx_cfg_req                     	: std_logic;
signal	tx_err_drop                    	: std_logic;
signal	tx_cfg_gnt                     	: std_logic;
		
signal	s_axis_tx_tready               	: std_logic;
signal	s_axis_tx_tdata                	: std_logic_vector((TRN_DT_BITS - 1) downto 0);
signal	s_axis_tx_tkeep                	: std_logic_vector((TRN_DT_BITS/8)-1 downto 0);
signal	s_axis_tx_tuser                	: std_logic_vector(3 downto 0);
signal	s_axis_tx_tlast                	: std_logic;
signal	s_axis_tx_tvalid               	: std_logic;
		
		-- RX
signal	rx_np_ok                       	: std_logic;
signal	rx_np_req                      	: std_logic;
signal	m_axis_rx_tdata                	: std_logic_vector((TRN_DT_BITS - 1) downto 0);
signal	m_axis_rx_tkeep                	: std_logic_vector((TRN_DT_BITS/8)-1 downto 0);
signal	m_axis_rx_tlast                	: std_logic;
signal	m_axis_rx_tvalid               	: std_logic;
signal	m_axis_rx_tready               	: std_logic;
signal	m_axis_rx_tuser                	: std_logic_vector(21 downto 0);
		
		-- Flow Control
signal	fc_cpld                        	: std_logic_vector(11 downto 0);
signal	fc_cplh                        	: std_logic_vector(7 downto 0);
signal	fc_npd                         	: std_logic_vector(11 downto 0);
signal	fc_nph                         	: std_logic_vector(7 downto 0);
signal	fc_pd                          	: std_logic_vector(11 downto 0);
signal	fc_ph                          	: std_logic_vector(7 downto 0);
signal	fc_sel                         	: std_logic_vector(2 downto 0);
		
		-- CFG
		
signal	cfg_err_cor                    	: std_logic;
signal	cfg_err_ur                     	: std_logic;
signal	cfg_err_ecrc                   	: std_logic;
signal	cfg_err_cpl_timeout            	: std_logic;
signal	cfg_err_cpl_unexpect           	: std_logic;
signal	cfg_err_cpl_abort              	: std_logic;
signal	cfg_err_atomic_egress_blocked  	: std_logic;
signal	cfg_err_internal_cor           	: std_logic;
signal	cfg_err_malformed              	: std_logic;
signal	cfg_err_mc_blocked             	: std_logic;
signal	cfg_err_poisoned               	: std_logic;
signal	cfg_err_norecovery             	: std_logic;
signal	cfg_err_acs                    	: std_logic;
signal	cfg_err_internal_uncor         	: std_logic;
signal	cfg_pm_halt_aspm_l0s           	: std_logic;
signal	cfg_pm_halt_aspm_l1            	: std_logic;
signal	cfg_pm_force_state_en          	: std_logic;
signal	cfg_pm_force_state             	: std_logic_vector( 1 downto 0);
signal	cfg_err_posted                 	: std_logic;
signal	cfg_err_locked                 	: std_logic;
signal	cfg_err_tlp_cpl_header         	: std_logic_vector(47 downto 0);
signal	cfg_err_cpl_rdy                	: std_logic;
signal	cfg_interrupt                  	: std_logic;
signal	cfg_interrupt_rdy              	: std_logic;
signal	cfg_interrupt_assert           	: std_logic;
signal	cfg_interrupt_di               	: std_logic_vector( 7 downto 0);
signal	cfg_interrupt_do               	: std_logic_vector( 7 downto 0);
signal	cfg_interrupt_mmenable         	: std_logic_vector( 2 downto 0);
signal	cfg_interrupt_msienable        	: std_logic;
signal	cfg_interrupt_msixenable       	: std_logic;
signal	cfg_interrupt_msixfm           	: std_logic;
signal	cfg_turnoff_ok                 	: std_logic;
signal	cfg_to_turnoff                 	: std_logic;
signal	cfg_trn_pending                	: std_logic;
signal	cfg_pm_wake                    	: std_logic;
signal	cfg_bus_number                 	: std_logic_vector( 7 downto 0);
signal	cfg_device_number              	: std_logic_vector( 4 downto 0);
signal	cfg_function_number            	: std_logic_vector( 2 downto 0);
signal	cfg_status                     	: std_logic_vector(15 downto 0);
signal	cfg_command                    	: std_logic_vector(15 downto 0);
signal	cfg_dstatus                    	: std_logic_vector(15 downto 0);
signal	cfg_dcommand                   	: std_logic_vector(15 downto 0);
signal	cfg_lstatus                    	: std_logic_vector(15 downto 0);
signal	cfg_lcommand                   	: std_logic_vector(15 downto 0);
signal	cfg_dcommand2                  	: std_logic_vector(15 downto 0);
signal	cfg_pcie_link_state            	: std_logic_vector( 2 downto 0);
		
signal	cfg_interrupt_stat             	: std_logic;
signal	cfg_pciecap_interrupt_msgnum   	: std_logic_vector( 4 downto 0);
		
signal	pl_directed_link_change        	: std_logic_vector( 1 downto 0);
signal	pl_ltssm_state                 	: std_logic_vector( 5 downto 0);
signal	pl_directed_link_width         	: std_logic_vector( 1 downto 0);
signal	pl_directed_link_speed         	: std_logic;
signal	pl_directed_link_auton         	: std_logic;
signal	pl_upstream_prefer_deemph      	: std_logic;
signal	pl_sel_lnk_width               	: std_logic_vector( 1 downto 0);
signal	pl_sel_lnk_rate                	: std_logic;
signal	pl_link_gen2_cap               	: std_logic;
signal	pl_link_partner_gen2_supported 	: std_logic;
signal	pl_initial_link_width          	: std_logic_vector( 2 downto 0);
signal	pl_link_upcfg_cap              	: std_logic;
signal	pl_lane_reversal_mode          	: std_logic_vector( 1 downto 0);
signal	pl_received_hot_rst            	: std_logic;
		
signal	cfg_err_aer_headerlog          	: std_logic_vector(127 downto 0);
signal	cfg_aer_interrupt_msgnum       	: std_logic_vector(4 downto 0);
signal	cfg_err_aer_headerlog_set      	: std_logic;
signal	cfg_aer_ecrc_check_en          	: std_logic;
signal	cfg_aer_ecrc_gen_en            	: std_logic;
		
signal	cfg_mgmt_di                    	: std_logic_vector(31 downto 0);
signal	cfg_mgmt_byte_en               	: std_logic_vector( 3 downto 0);
signal	cfg_mgmt_dwaddr                	: std_logic_vector( 9 downto 0);
signal	cfg_mgmt_wr_en                 	: std_logic;
signal	cfg_mgmt_rd_en                 	: std_logic;
signal	cfg_mgmt_wr_readonly           	: std_logic;
		
signal	cfg_dsn                        	: std_logic_vector(63 downto 0);


-- MIG
signal	MIG_CLK 				: std_logic;							-- PORT MIG_CLK = MIG_CLK, DIR = I
signal	app_wdf_wren 			: std_logic;							-- PORT app_wdf_wren = app_wdf_wren, DIR = O
signal	app_wdf_data 			: std_logic_vector(MIG_DT_BITS-1 downto 0);		-- PORT app_wdf_data = app_wdf_data, DIR = O, VEC = [255:0]
signal	app_wdf_mask 			: std_logic_vector(MIG_DT_BITS/8-1 downto 0);		-- PORT app_wdf_mask = app_wdf_mask, DIR = O, VEC = [31:0]
signal	app_wdf_end 			: std_logic;							-- PORT app_wdf_end = app_wdf_end, DIR = O
signal	app_addr 				: std_logic_vector(31 downto 0) := X"00000000";		-- PORT app_addr = app_addr, DIR = O, VEC = [26:0]
signal	app_cmd 				: std_logic_vector(2 downto 0);			-- PORT app_cmd = app_cmd, DIR = O, VEC = [2:0]
signal	app_en 					: std_logic;							-- PORT app_en = app_en, DIR = O
signal	app_rdy 				: std_logic;							-- PORT app_rdy = app_rdy, DIR = I
signal	app_wdf_rdy 			: std_logic;							-- PORT app_wdf_rdy = app_wdf_rdy, DIR = I
signal	app_rd_data 			: std_logic_vector(MIG_DT_BITS-1 downto 0);		-- PORT app_rd_data = app_rd_data, DIR = I, VEC = [255:0]
signal	app_rd_data_end 		: std_logic;							-- PORT app_rd_data_end = app_rd_data_end, DIR = I
signal	app_rd_data_valid 		: std_logic;							-- PORT app_rd_data_valid = app_rd_data_valid, DIR = I
signal	phy_init_done 			: std_logic;
signal	sys_rst					: std_logic;
signal	init_calib_complete 	: std_logic;

signal	user_gpio_out_00		: std_logic_vector(31 downto 0);
signal	user_gpio_out_01		: std_logic_vector(31 downto 0);
signal	user_gpio_out_02		: std_logic_vector(31 downto 0);
signal	user_gpio_out_03		: std_logic_vector(31 downto 0);
signal	user_gpio_in_00			: std_logic_vector(31 downto 0);
signal	user_gpio_in_01			: std_logic_vector(31 downto 0);
signal	user_gpio_in_02			: std_logic_vector(31 downto 0);
signal	user_gpio_in_03			: std_logic_vector(31 downto 0);

signal	user_mwr_ad				: std_logic_vector(31 downto 6);
signal	user_mwr_ad_en			: std_logic;
signal	user_mwr_ad_afull		: std_logic;
signal	user_mwr_dt				: std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
signal	user_mwr_dt_en			: std_logic;

signal	user_mrd_ad				: std_logic_vector(31 downto 6);
signal	user_mrd_ad_en			: std_logic;
signal	user_mrd_ad_afull		: std_logic;
signal	user_mrd_dt_en			: std_logic;
signal	user_mrd_dt				: std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
signal	user_mrd_dt_count		: std_logic_vector(UMRD_ABITS_AD-1 downto 0);

begin

	--------------------------------------------------------------------
	-- Zbg
	-- FAN
	--------------------------------------------------------------------
	RST			<= CPU_RESET;
	XRST		<= not RST;
	SM_FAN_PWM	<= '1';

	--------------------------------------------------------------------
	-- RUN_LED  500msœ_
	--------------------------------------------------------------------
	process(user_clk) begin
		if(user_clk'event and user_clk = '1') then
			if(timer_1ms_p = '1')then
				if(timer_500ms = 500-1)then
					RUN_LED_buf		<= not RUN_LED_buf;
					timer_500ms 	<= (others => '0');
				else
					timer_500ms 	<= timer_500ms + 1;
				end if;
			end if;
		end if;
	end process;
	--------------------------------------------------------------------
	-- STATUS_LED
	--------------------------------------------------------------------
	GPIO_LED_0_LS			<= RUN_LED_buf	;								-- 500msœ_
	GPIO_LED_7_LS			<= pl_sel_lnk_rate;								-- _GEN2
	GPIO_LED_6_LS			<= pl_sel_lnk_width(1) and pl_sel_lnk_width(0);	-- _8lane

--########################################################################################
--                              instantiate sub-blocks
--########################################################################################
	u_xilinx_pcie_2_1_ep_7x : xilinx_pcie_2_1_ep_7x 
	port map(
		pci_exp_txp                   	=> pci_exp_txp  			,	--	: out std_logic_vector(7 downto 0);
		pci_exp_txn                   	=> pci_exp_txn  			,	--	: out std_logic_vector(7 downto 0);
		pci_exp_rxp                   	=> pci_exp_rxp  			,	--	: in std_logic_vector(7 downto 0);
		pci_exp_rxn                   	=> pci_exp_rxn  			,	--	: in std_logic_vector(7 downto 0);
		
		sys_clk_p                     	=> sys_clk_p	 			,	--	: in std_logic;
		sys_clk_n                     	=> sys_clk_n   				,	--	: in std_logic;
		sys_rst_n                     	=> sys_rst_n	  			,	--	: in std_logic;
		
		
		--  component pcie_app_7x
		user_clk_out                  	=> user_clk 				,	--	: out std_logic;													
		user_reset_out                	=> user_reset	 			,	--	: out std_logic;                                                 
		user_lnk_up_out               	=> user_lnk_up		        ,	--	: out std_logic;                                                 
		s_axis_tx_tdata               	=> s_axis_tx_tdata          ,	--	: in std_logic_vector((C_DATA_WIDTH - 1) downto 0);              
		s_axis_tx_tkeep               	=> s_axis_tx_tkeep          ,	--	: in std_logic_vector((C_DATA_WIDTH / 8 - 1) downto 0);          
		s_axis_tx_tlast               	=> s_axis_tx_tlast          ,	--	: in std_logic;                                                  
		s_axis_tx_tvalid              	=> s_axis_tx_tvalid         ,	--	: in std_logic;                                                  
		s_axis_tx_tready              	=> s_axis_tx_tready         ,	--	: out std_logic;                                                 
		s_axis_tx_tuser               	=> s_axis_tx_tuser          ,	--	: in std_logic_vector(3 downto 0);                               
		tx_cfg_req                    	=> tx_cfg_req               ,	--	: out std_logic;                                                 
		tx_cfg_gnt                    	=> tx_cfg_gnt               ,	--	: in std_logic;                                                  
		tx_err_drop                   	=> tx_err_drop              ,	--	: out std_logic;                                                 
		tx_buf_av                     	=> tx_buf_av                ,	--	: out std_logic_vector(5 downto 0);                              
		m_axis_rx_tdata               	=> m_axis_rx_tdata          ,	--	: out std_logic_vector((C_DATA_WIDTH - 1) downto 0);             
		m_axis_rx_tkeep               	=> m_axis_rx_tkeep          ,	--	: out std_logic_vector((C_DATA_WIDTH / 8 - 1) downto 0);         
		m_axis_rx_tlast               	=> m_axis_rx_tlast          ,	--	: out std_logic;                                                 
		m_axis_rx_tvalid              	=> m_axis_rx_tvalid         ,	--	: out std_logic;                                                 
		m_axis_rx_tready              	=> m_axis_rx_tready         ,	--	: in std_logic;                                                  
		m_axis_rx_tuser               	=> m_axis_rx_tuser          ,	--	: out std_logic_vector(21 downto 0);                             
		rx_np_ok                      	=> rx_np_ok                 ,	--	: in std_logic;                                                  
		rx_np_req                     	=> rx_np_req                ,	--	: in std_logic;                                                  
		fc_nph                        	=> fc_nph                   ,	--	: out std_logic_vector(7 downto 0);                              
		fc_npd                        	=> fc_npd                   ,	--	: out std_logic_vector(11 downto 0);                             
		fc_ph                         	=> fc_ph                    ,	--	: out std_logic_vector(7 downto 0);                              
		fc_pd                         	=> fc_pd                    ,	--	: out std_logic_vector(11 downto 0);                             
		fc_cplh                       	=> fc_cplh                  ,	--	: out std_logic_vector(7 downto 0);                              
		fc_cpld                       	=> fc_cpld                  ,	--	: out std_logic_vector(11 downto 0);                             
		fc_sel                        	=> fc_sel                   ,	--	: in std_logic_vector(2 downto 0);                               
		cfg_err_cor                   	=> cfg_err_cor              ,	--	: in std_logic;                                                  
		cfg_err_ur                    	=> cfg_err_ur               ,	--	: in std_logic;                                                  
		cfg_err_cpl_rdy               	=> cfg_err_cpl_rdy          ,	--	: out std_logic;                                                 
		cfg_err_ecrc                  	=> cfg_err_ecrc             ,	--	: in std_logic;                                                  
		cfg_err_cpl_timeout           	=> cfg_err_cpl_timeout      ,	--	: in std_logic;                                                  
		cfg_err_cpl_abort             	=> cfg_err_cpl_abort        ,	--	: in std_logic;                                                  
		cfg_err_cpl_unexpect          	=> cfg_err_cpl_unexpect     ,	--	: in std_logic;                                                  
		cfg_err_posted                	=> cfg_err_posted           ,	--	: in std_logic;                                                  
		cfg_err_locked                	=> cfg_err_locked           ,	--	: in std_logic;                                                  
		cfg_err_atomic_egress_blocked 	=> cfg_err_atomic_egress_blocked	,	--	: in std_logic;                                                  
		cfg_err_internal_cor          	=> cfg_err_internal_cor     ,	--	: in std_logic;                                                  
		cfg_err_malformed             	=> cfg_err_malformed        ,	--	: in std_logic;                                                  
		cfg_err_mc_blocked            	=> cfg_err_mc_blocked       ,	--	: in std_logic;                                                  
		cfg_err_poisoned              	=> cfg_err_poisoned         ,	--	: in std_logic;                                                  
		cfg_err_norecovery            	=> cfg_err_norecovery       ,	--	: in std_logic;                                                  
		cfg_err_acs                   	=> cfg_err_acs              ,	--	: in std_logic;                                                  
		cfg_err_internal_uncor        	=> cfg_err_internal_uncor   ,	--	: in std_logic;                                                  
		cfg_err_aer_headerlog         	=> cfg_err_aer_headerlog    ,	--	: in std_logic_vector(127 downto 0);                             
		cfg_aer_interrupt_msgnum      	=> cfg_aer_interrupt_msgnum ,	--	: in std_logic_vector(4 downto 0);                               
		cfg_err_aer_headerlog_set     	=> cfg_err_aer_headerlog_set,	--	: out  std_logic;                                                
		cfg_aer_ecrc_check_en         	=> cfg_aer_ecrc_check_en    ,	--	: out  std_logic;                                                
		cfg_aer_ecrc_gen_en           	=> cfg_aer_ecrc_gen_en      ,	--	: out  std_logic;                                                
		cfg_interrupt                 	=> cfg_interrupt            ,	--	: in std_logic;                                                  
		cfg_interrupt_rdy             	=> cfg_interrupt_rdy        ,	--	: out std_logic;                                                 
		cfg_interrupt_assert          	=> cfg_interrupt_assert     ,	--	: in std_logic;                                                  
		cfg_interrupt_di              	=> cfg_interrupt_di         ,	--	: in std_logic_vector(7 downto 0);                               
		cfg_interrupt_do              	=> cfg_interrupt_do         ,	--	: out  std_logic_vector(7 downto 0);                             
		cfg_interrupt_mmenable        	=> cfg_interrupt_mmenable   ,	--	: out  std_logic_vector(2 downto 0);                             
		cfg_interrupt_msienable       	=> cfg_interrupt_msienable  ,	--	: out  std_logic;                                                
		cfg_interrupt_msixenable      	=> cfg_interrupt_msixenable ,	--	: out  std_logic;                                                
		cfg_interrupt_msixfm          	=> cfg_interrupt_msixfm     ,	--	: out  std_logic;                                                
		cfg_interrupt_stat            	=> cfg_interrupt_stat       ,	--	: in std_logic;                                                  
		cfg_pciecap_interrupt_msgnum  	=> cfg_pciecap_interrupt_msgnum		,	--	: in std_logic_vector( 4 downto 0);                              
		cfg_turnoff_ok                	=> cfg_turnoff_ok           ,	--	: in std_logic;                                                  
		cfg_to_turnoff                	=> cfg_to_turnoff           ,	--	: out std_logic;                                                 
		cfg_pm_wake                   	=> cfg_pm_wake              ,	--	: in std_logic;                                                  
		cfg_pm_force_state_en         	=> cfg_pm_force_state_en    ,	--	: in std_logic;                                                  
		cfg_pm_force_state            	=> cfg_pm_force_state       ,	--	: in std_logic_vector( 1 downto 0);                              
		cfg_pm_halt_aspm_l0s          	=> cfg_pm_halt_aspm_l0s     ,	--	: in std_logic;                                                  
		cfg_pm_halt_aspm_l1           	=> cfg_pm_halt_aspm_l1      ,	--	: in std_logic;                                                  
		cfg_pcie_link_state           	=> cfg_pcie_link_state      ,	--	: out std_logic_vector(2 downto 0);                              
		cfg_trn_pending               	=> cfg_trn_pending          ,	--	: in std_logic;                                                  
		cfg_err_tlp_cpl_header        	=> cfg_err_tlp_cpl_header   ,	--	: in std_logic_vector(47 downto 0);                              
		cfg_bus_number                	=> cfg_bus_number           ,	--	: out std_logic_vector(7 downto 0);                              
		cfg_device_number             	=> cfg_device_number        ,	--	: out std_logic_vector(4 downto 0);                              
		cfg_function_number           	=> cfg_function_number      ,	--	: out std_logic_vector(2 downto 0);                              
		cfg_status                    	=> cfg_status               ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_command                   	=> cfg_command              ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_dstatus                   	=> cfg_dstatus              ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_dcommand                  	=> cfg_dcommand             ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_lstatus                   	=> cfg_lstatus              ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_lcommand                  	=> cfg_lcommand             ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_dcommand2                 	=> cfg_dcommand2            ,	--	: out std_logic_vector(15 downto 0);                             
		cfg_mgmt_di                   	=> cfg_mgmt_di              ,	--	: in std_logic_vector(31 downto 0);                              
		cfg_mgmt_byte_en              	=> cfg_mgmt_byte_en         ,	--	: in std_logic_vector( 3 downto 0);                              
		cfg_mgmt_dwaddr               	=> cfg_mgmt_dwaddr          ,	--	: in std_logic_vector( 9 downto 0);                              
		cfg_mgmt_wr_en                	=> cfg_mgmt_wr_en           ,	--	: in std_logic;                                                  
		cfg_mgmt_rd_en                	=> cfg_mgmt_rd_en           ,	--	: in std_logic;                                                  
		cfg_mgmt_wr_readonly          	=> cfg_mgmt_wr_readonly     ,	--	: in std_logic;                                                  
		pl_directed_link_change       	=> pl_directed_link_change  ,	--	: in std_logic_vector(1 downto 0);                               
		pl_ltssm_state                	=> pl_ltssm_state           ,	--	: out std_logic_vector(5 downto 0);                              
		pl_directed_link_width        	=> pl_directed_link_width   ,	--	: in std_logic_vector(1 downto 0);                               
		pl_directed_link_speed        	=> pl_directed_link_speed   ,	--	: in std_logic;                                                  
		pl_directed_link_auton        	=> pl_directed_link_auton   ,	--	: in std_logic;                                                  
		pl_upstream_prefer_deemph     	=> pl_upstream_prefer_deemph,	--	: in std_logic;                                                  
		pl_sel_lnk_width              	=> pl_sel_lnk_width         ,	--	: out std_logic_vector(1 downto 0);                              
		pl_sel_lnk_rate               	=> pl_sel_lnk_rate          ,	--	: out std_logic;                                                 
		pl_link_gen2_cap              	=> pl_link_gen2_cap         ,	--	: out std_logic;                                                 
		pl_link_partner_gen2_supported	=> pl_link_partner_gen2_supported	,	--	: out std_logic;                                                 
		pl_initial_link_width         	=> pl_initial_link_width    ,	--	: out std_logic_vector(2 downto 0);                              
		pl_link_upcfg_cap             	=> pl_link_upcfg_cap        ,	--	: out std_logic;                                                 
		pl_lane_reversal_mode         	=> pl_lane_reversal_mode    ,	--	: out std_logic_vector(1 downto 0);                              
		pl_received_hot_rst           	=> pl_received_hot_rst      ,	--	: out std_logic;                                                 
		cfg_dsn                       	=> cfg_dsn                  	--	: in std_logic_vector(63 downto 0)                               
	);

	u_iSKY_PCIe : iSKY_PCIe
	generic map (
		TRN_DT_BITS						=> TRN_DT_BITS						,	--	: INTEGER := 64;
		MIG_DT_BITS						=> MIG_DT_BITS						,	--	: integer range 256 to 512 := 256;
		MIG_DB_CYCLE					=> MIG_DB_CYCLE						,	--	: integer range 1 to 2 := 1;			-- app_xdf ]NbN

		UMWT_TRN_DT_BITS				=> UMWT_TRN_DT_BITS					,	--	: INTEGER := 64;
		UMWT_TRN_MK_BITS				=> UMWT_TRN_DT_BITS/32				,	--	: INTEGER := 2;

		UMRD_ABITS_AD					=> UMRD_ABITS_AD					,	--	: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
		UMRD_TRN_DT_BITS				=> UMRD_TRN_DT_BITS						--	: INTEGER := 64
	)
	port map (
--<< PCI Express >>
		user_clk               			=> user_clk 						,	--	: in	std_logic;
		user_reset             			=> user_reset	 					,	--	: in  	std_logic;
		user_lnk_up            			=> user_lnk_up		 				,	--	: in  	std_logic;

		-- Tx
		tx_buf_av                      	=> tx_buf_av                      	,	--	: in  	std_logic_vector(5 downto 0);
		tx_cfg_req                     	=> tx_cfg_req                     	,	--	: in  	std_logic;
		tx_err_drop                    	=> tx_err_drop                    	,	--	: in  	std_logic;
		tx_cfg_gnt                     	=> tx_cfg_gnt                     	,	--	: out 	std_logic;
		                                   
		s_axis_tx_tready               	=> s_axis_tx_tready               	,	--	: in  	std_logic;
		s_axis_tx_tdata                	=> s_axis_tx_tdata                	,	--	: out 	std_logic_vector((TRN_DT_BITS - 1) downto 0);
		s_axis_tx_tkeep                	=> s_axis_tx_tkeep                	,	--	: out 	std_logic_vector((TRN_DT_BITS/8)-1 downto 0);
		s_axis_tx_tuser                	=> s_axis_tx_tuser                	,	--	: out 	std_logic_vector(3 downto 0);
		s_axis_tx_tlast                	=> s_axis_tx_tlast                	,	--	: out 	std_logic;
		s_axis_tx_tvalid               	=> s_axis_tx_tvalid               	,	--	: out 	std_logic;
		                                   
		-- RX                              -- RX
		rx_np_ok                       	=> rx_np_ok                       	,	--	: out 	std_logic;
		rx_np_req                      	=> rx_np_req                      	,	--	: out 	std_logic;
		m_axis_rx_tdata                	=> m_axis_rx_tdata                	,	--	: in 	std_logic_vector((TRN_DT_BITS - 1) downto 0);
		m_axis_rx_tkeep                	=> m_axis_rx_tkeep                	,	--	: in 	std_logic_vector((TRN_DT_BITS/8)-1 downto 0);
		m_axis_rx_tlast                	=> m_axis_rx_tlast                	,	--	: in  	std_logic;
		m_axis_rx_tvalid               	=> m_axis_rx_tvalid               	,	--	: in  	std_logic;
		m_axis_rx_tready               	=> m_axis_rx_tready               	,	--	: out 	std_logic;
		m_axis_rx_tuser                	=> m_axis_rx_tuser                	,	--	: in 	std_logic_vector(21 downto 0);
		                                   
		-- Flow Control                    -- Flow Control
		fc_cpld                        	=> fc_cpld                        	,	--	: in  	std_logic_vector(11 downto 0);
		fc_cplh                        	=> fc_cplh                        	,	--	: in  	std_logic_vector(7 downto 0);
		fc_npd                         	=> fc_npd                         	,	--	: in  	std_logic_vector(11 downto 0);
		fc_nph                         	=> fc_nph                         	,	--	: in  	std_logic_vector(7 downto 0);
		fc_pd                          	=> fc_pd                          	,	--	: in  	std_logic_vector(11 downto 0);
		fc_ph                          	=> fc_ph                          	,	--	: in  	std_logic_vector(7 downto 0);
		fc_sel                         	=> fc_sel                         	,	--	: out 	std_logic_vector(2 downto 0);
		                                   
		-- CFG                             -- CFG
		                                   
		cfg_err_cor                    	=> cfg_err_cor                    	,	--	: out 	std_logic;
		cfg_err_ur                     	=> cfg_err_ur                     	,	--	: out 	std_logic;
		cfg_err_ecrc                   	=> cfg_err_ecrc                   	,	--	: out 	std_logic;
		cfg_err_cpl_timeout            	=> cfg_err_cpl_timeout            	,	--	: out 	std_logic;
		cfg_err_cpl_unexpect           	=> cfg_err_cpl_unexpect           	,	--	: out 	std_logic;
		cfg_err_cpl_abort              	=> cfg_err_cpl_abort              	,	--	: out 	std_logic;
		cfg_err_atomic_egress_blocked  	=> cfg_err_atomic_egress_blocked  	,	--	: out 	std_logic;
		cfg_err_internal_cor           	=> cfg_err_internal_cor           	,	--	: out 	std_logic;
		cfg_err_malformed              	=> cfg_err_malformed              	,	--	: out 	std_logic;
		cfg_err_mc_blocked             	=> cfg_err_mc_blocked             	,	--	: out 	std_logic;
		cfg_err_poisoned               	=> cfg_err_poisoned               	,	--	: out 	std_logic;
		cfg_err_norecovery             	=> cfg_err_norecovery             	,	--	: out 	std_logic;
		cfg_err_acs                    	=> cfg_err_acs                    	,	--	: out 	std_logic;
		cfg_err_internal_uncor         	=> cfg_err_internal_uncor         	,	--	: out 	std_logic;
		cfg_pm_halt_aspm_l0s           	=> cfg_pm_halt_aspm_l0s           	,	--	: out 	std_logic;
		cfg_pm_halt_aspm_l1            	=> cfg_pm_halt_aspm_l1            	,	--	: out 	std_logic;
		cfg_pm_force_state_en          	=> cfg_pm_force_state_en          	,	--	: out 	std_logic;
		cfg_pm_force_state             	=> cfg_pm_force_state             	,	--	: out 	std_logic_vector( 1 downto 0);
		cfg_err_posted                 	=> cfg_err_posted                 	,	--	: out 	std_logic;
		cfg_err_locked                 	=> cfg_err_locked                 	,	--	: out 	std_logic;
		cfg_err_tlp_cpl_header         	=> cfg_err_tlp_cpl_header         	,	--	: out 	std_logic_vector(47 downto 0);
		cfg_err_cpl_rdy                	=> cfg_err_cpl_rdy                	,	--	: in  	std_logic;
		cfg_interrupt                  	=> cfg_interrupt                  	,	--	: out 	std_logic;
		cfg_interrupt_rdy              	=> cfg_interrupt_rdy              	,	--	: in  	std_logic;
		cfg_interrupt_assert           	=> cfg_interrupt_assert           	,	--	: out 	std_logic;
		cfg_interrupt_di               	=> cfg_interrupt_di               	,	--	: out 	std_logic_vector( 7 downto 0);
		cfg_interrupt_do               	=> cfg_interrupt_do               	,	--	: in  	std_logic_vector( 7 downto 0);
		cfg_interrupt_mmenable         	=> cfg_interrupt_mmenable         	,	--	: in  	std_logic_vector( 2 downto 0);
		cfg_interrupt_msienable        	=> cfg_interrupt_msienable        	,	--	: in  	std_logic;
		cfg_interrupt_msixenable       	=> cfg_interrupt_msixenable       	,	--	: in  	std_logic;
		cfg_interrupt_msixfm           	=> cfg_interrupt_msixfm           	,	--	: in  	std_logic;
		cfg_turnoff_ok                 	=> cfg_turnoff_ok                 	,	--	: out 	std_logic;
		cfg_to_turnoff                 	=> cfg_to_turnoff                 	,	--	: in  	std_logic;
		cfg_trn_pending                	=> cfg_trn_pending                	,	--	: out 	std_logic;
		cfg_pm_wake                    	=> cfg_pm_wake                    	,	--	: out 	std_logic;
		cfg_bus_number                 	=> cfg_bus_number                 	,	--	: in  	std_logic_vector( 7 downto 0);
		cfg_device_number              	=> cfg_device_number              	,	--	: in  	std_logic_vector( 4 downto 0);
		cfg_function_number            	=> cfg_function_number            	,	--	: in  	std_logic_vector( 2 downto 0);
		cfg_status                     	=> cfg_status                     	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_command                    	=> cfg_command                    	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_dstatus                    	=> cfg_dstatus                    	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_dcommand                   	=> cfg_dcommand                   	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_lstatus                    	=> cfg_lstatus                    	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_lcommand                   	=> cfg_lcommand                   	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_dcommand2                  	=> cfg_dcommand2                  	,	--	: in  	std_logic_vector(15 downto 0);
		cfg_pcie_link_state            	=> cfg_pcie_link_state            	,	--	: in  	std_logic_vector( 2 downto 0);
		                                   
		cfg_interrupt_stat             	=> cfg_interrupt_stat             	,	--	: out 	std_logic;
		cfg_pciecap_interrupt_msgnum   	=> cfg_pciecap_interrupt_msgnum   	,	--	: out 	std_logic_vector( 4 downto 0);
		                                   
		pl_directed_link_change        	=> pl_directed_link_change        	,	--	: out 	std_logic_vector( 1 downto 0);
		pl_ltssm_state                 	=> pl_ltssm_state                 	,	--	: in  	std_logic_vector( 5 downto 0);
		pl_directed_link_width         	=> pl_directed_link_width         	,	--	: out 	std_logic_vector( 1 downto 0);
		pl_directed_link_speed         	=> pl_directed_link_speed         	,	--	: out 	std_logic;
		pl_directed_link_auton         	=> pl_directed_link_auton         	,	--	: out 	std_logic;
		pl_upstream_prefer_deemph      	=> pl_upstream_prefer_deemph      	,	--	: out 	std_logic;
		pl_sel_lnk_width               	=> pl_sel_lnk_width               	,	--	: in  	std_logic_vector( 1 downto 0);
		pl_sel_lnk_rate                	=> pl_sel_lnk_rate                	,	--	: in  	std_logic;
		pl_link_gen2_cap               	=> pl_link_gen2_cap               	,	--	: in  	std_logic;
		pl_link_partner_gen2_supported 	=> pl_link_partner_gen2_supported 	,	--	: in  	std_logic;
		pl_initial_link_width          	=> pl_initial_link_width          	,	--	: in  	std_logic_vector( 2 downto 0);
		pl_link_upcfg_cap              	=> pl_link_upcfg_cap              	,	--	: in  	std_logic;
		pl_lane_reversal_mode          	=> pl_lane_reversal_mode          	,	--	: in  	std_logic_vector( 1 downto 0);
		pl_received_hot_rst            	=> pl_received_hot_rst            	,	--	: in  	std_logic;
		                                   
		cfg_err_aer_headerlog          	=> cfg_err_aer_headerlog          	,	--	: out 	std_logic_vector(127 downto 0);
		cfg_aer_interrupt_msgnum       	=> cfg_aer_interrupt_msgnum       	,	--	: out 	std_logic_vector(4 downto 0);
		cfg_err_aer_headerlog_set      	=> cfg_err_aer_headerlog_set      	,	--	: in  	std_logic;
		cfg_aer_ecrc_check_en          	=> cfg_aer_ecrc_check_en          	,	--	: in  	std_logic;
		cfg_aer_ecrc_gen_en            	=> cfg_aer_ecrc_gen_en            	,	--	: in  	std_logic;
		                                   
		cfg_mgmt_di                    	=> cfg_mgmt_di                    	,	--	: out 	std_logic_vector(31 downto 0);
		cfg_mgmt_byte_en               	=> cfg_mgmt_byte_en               	,	--	: out 	std_logic_vector( 3 downto 0);
		cfg_mgmt_dwaddr                	=> cfg_mgmt_dwaddr                	,	--	: out 	std_logic_vector( 9 downto 0);
		cfg_mgmt_wr_en                 	=> cfg_mgmt_wr_en                 	,	--	: out 	std_logic;
		cfg_mgmt_rd_en                 	=> cfg_mgmt_rd_en                 	,	--	: out 	std_logic;
		cfg_mgmt_wr_readonly           	=> cfg_mgmt_wr_readonly           	,	--	: out 	std_logic;
		                                   
		cfg_dsn                        	=> cfg_dsn                        	,	--	: out 	std_logic_vector(63 downto 0);
-- << MIG >>
		mig_clk          				=> MIG_CLK							,	--	: in  std_logic;
--		mig_clk          				=> user_clk							,	--	: in  std_logic;
		app_addr      					=> app_addr				 			,	--	: out	std_logic_vector(26 downto 0);	-- 
		app_cmd       					=> app_cmd       		 			,	--	: out	std_logic_vector(2 downto 0);	-- 
		app_en        					=> app_en        		 			,	--	: out	std_logic;						-- 
		app_rdy       					=> app_rdy       					,	--	: in	std_logic;						-- 
                                                               
		app_wdf_wren  					=> app_wdf_wren  					,	--	: out	std_logic;						-- 
		app_wdf_data  					=> app_wdf_data  					,	--	: out	std_logic_vector(255 downto 0);	-- 
		app_wdf_mask					=> app_wdf_mask						,	--	: out	std_logic_vector(31 downto 0);
		app_wdf_end   					=> app_wdf_end   					,	--	: out	std_logic;						-- 
		app_wdf_rdy   					=> app_wdf_rdy   					,	--	: in	std_logic;						-- 
                                                               
		app_rd_data   					=> app_rd_data   					,	--	: in	std_logic_vector(255 downto 0);	-- 
		app_rd_data_end 				=> app_rd_data_end 					,	--	: in	std_logic;						-- 
		app_rd_data_valid				=> app_rd_data_valid				,	--	: in	std_logic;						-- 
                                
-- << USER IF - GPIO >>
		user_gpio_out_00				=> user_gpio_out_00					,	--	: out	std_logic_vector(31 downto 0);
		user_gpio_out_01				=> user_gpio_out_01					,	--	: out	std_logic_vector(31 downto 0);
		user_gpio_out_02				=> user_gpio_out_02					,	--	: out	std_logic_vector(31 downto 0);
		user_gpio_out_03				=> user_gpio_out_03					,	--	: out	std_logic_vector(31 downto 0);
		user_gpio_in_00					=> user_gpio_in_00					,	--	: in	std_logic_vector(31 downto 0);
		user_gpio_in_01					=> user_gpio_in_01					,	--	: in	std_logic_vector(31 downto 0);
		user_gpio_in_02					=> X"00000000"						,	--	: in	std_logic_vector(31 downto 0);
		user_gpio_in_03					=> X"00000000"						,	--	: in	std_logic_vector(31 downto 0);
-- << USER IF - FIFO(MIG)>>
		user_mwr_clk					=> CLK200MHz						,	--	: in	std_logic;
		user_mwr_ad						=> user_mwr_ad						,	--	: in	std_logic_vector(31 downto 6);
		user_mwr_ad_en					=> user_mwr_ad_en					,	--	: in	std_logic;
		user_mwr_ad_afull				=> user_mwr_ad_afull				,	--	: out	std_logic;
		user_mwr_dt						=> user_mwr_dt						,	--	: in	std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
		user_mwr_dt_en					=> user_mwr_dt_en					,	--	: in	std_logic;
		user_mwr_mask					=> (others => '0')					,	--	: in	std_logic_vector(UMWT_TRN_MK_BITS-1 downto 0);
                                
		user_mrd_clk					=> CLK200MHz						,	--	: in	std_logic;
		user_mrd_ad						=> user_mrd_ad						,	--	: in	std_logic_vector(31 downto 6);
		user_mrd_ad_en					=> user_mrd_ad_en					,	--	: in	std_logic;
		user_mrd_ad_afull				=> user_mrd_ad_afull				,	--	: out	std_logic;
		user_mrd_dt_en					=> user_mrd_dt_en					,	--	: in	std_logic;
		user_mrd_dt						=> user_mrd_dt						,	--	: out 	std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
		user_mrd_dt_count				=> user_mrd_dt_count				,	--	: out 	std_logic_vector(UMRD_ABITS_AD-1 downto 0);
-- << 䃌WX^ANZX >>
		plbc_regc_ad					=> "0000000"						,	--	: in	std_logic_vector(6 downto 0);	-- AhX(2rbg͖)
		plbc_regc_wd					=> X"00000000"						,	--	: in	std_logic_vector(31 downto 0);	-- 䃉Cgf[^
		plbc_regc_wp					=> '0'								,	--	: in	std_logic;						-- 䃉CgpX
		plbc_regc_rd					=> OPEN									--	: out	std_logic_vector(31 downto 0)	-- 䃊[hf[^
	);

	u_timer_cnt : timer_cnt
	generic map (
		CLK_SPEED_MHZ			=> 300									--	: INTEGER := 400						-- CLKXs[h(400MHz)
	)
	port map (
		-- RESET / CLK
		RST						=> '0'								,	--	: in	std_logic;						-- 1:Zbg
		CLK						=> user_clk							,	--	: in	std_logic;						-- 1:Zbg
		-- ^C}o̓pX        
		timer_1us_p				=> OPEN								,	--	: out	std_logic;						-- 1us^C}o̓pX
		timer_1ms_p				=> timer_1ms_p						,	--	: out	std_logic;						-- 1ms^C}o̓pX
		timer_1s_p				=> timer_1s_p							--	: out	std_logic						-- 1s ^C}o̓pX
	);

	u_mig : mig_7series_v1_5
	port map(
		-- Inouts     
		ddr3_dq       			=> ddr3_dq       					,	--	: inout std_logic_vector(DQ_WIDTH-1 downto 0);
		ddr3_dqs_p    			=> ddr3_dqs_p    					,	--	: inout std_logic_vector(DQS_WIDTH-1 downto 0);
		ddr3_dqs_n    			=> ddr3_dqs_n    					,	--	: inout std_logic_vector(DQS_WIDTH-1 downto 0);
		-- Outputs                 
		ddr3_addr     			=> ddr3_addr     					,	--	: out   std_logic_vector(ROW_WIDTH-1 downto 0);
		ddr3_ba       			=> ddr3_ba       					,	--	: out   std_logic_vector(BANK_WIDTH-1 downto 0);
		ddr3_ras_n    			=> ddr3_ras_n    					,	--	: out   std_logic;
		ddr3_cas_n    			=> ddr3_cas_n    					,	--	: out   std_logic;
		ddr3_we_n     			=> ddr3_we_n     					,	--	: out   std_logic;
		ddr3_reset_n  			=> ddr3_reset_n  					,	--	: out   std_logic;
		ddr3_ck_p     			=> ddr3_ck_p     					,	--	: out   std_logic_vector(CK_WIDTH-1 downto 0);
		ddr3_ck_n     			=> ddr3_ck_n     					,	--	: out   std_logic_vector(CK_WIDTH-1 downto 0);
		ddr3_cke      			=> ddr3_cke      					,	--	: out   std_logic_vector(CKE_WIDTH-1 downto 0);
		ddr3_cs_n     			=> ddr3_cs_n     					,	--	: out   std_logic_vector(CS_WIDTH*nCS_PER_RANK-1 downto 0);
		ddr3_dm       			=> ddr3_dm       					,	--	: out   std_logic_vector(DM_WIDTH-1 downto 0);
		ddr3_odt      			=> ddr3_odt      					,	--	: out   std_logic_vector(ODT_WIDTH-1 downto 0);
   		-- Inputs
   		-- Differential system clocks
   		sys_clk_p               => SYSCLK_P 						,	--	: in    std_logic;
   		sys_clk_n               => SYSCLK_N       					,	--	: in    std_logic;
   		-- Single-ended iodelayctrl clk (reference clock)
--   	clk_ref_i				=>										--	: in    std_logic;
		-- user interface signals  
		app_addr      			=> app_addr(28-1 downto 0)      	,	--	: in    std_logic_vector(ADDR_WIDTH-1 downto 0);
		app_cmd       			=> app_cmd       					,	--	: in    std_logic_vector(2 downto 0);
		app_en        			=> app_en        					,	--	: in    std_logic;
		app_wdf_data  			=> app_wdf_data  					,	--	: in    std_logic_vector((nCK_PER_CLK*2*PAYLOAD_WIDTH)-1 downto 0);
		app_wdf_end   			=> app_wdf_end   					,	--	: in    std_logic;
		app_wdf_mask  			=> (others => '0')  				,	--	: in    std_logic_vector((nCK_PER_CLK*2*PAYLOAD_WIDTH)/8-1 downto 0)  ;
		app_wdf_wren  			=> app_wdf_wren  					,	--	: in    std_logic;
		app_rd_data   			=> app_rd_data   					,	--	: out   std_logic_vector((nCK_PER_CLK*2*PAYLOAD_WIDTH)-1 downto 0);
		app_rd_data_end   		=> app_rd_data_end   				,	--	: out   std_logic;
		app_rd_data_valid 		=> app_rd_data_valid 				,	--	: out   std_logic;
		app_rdy         		=> app_rdy         					,	--	: out   std_logic;
		app_wdf_rdy     		=> app_wdf_rdy     					,	--	: out   std_logic;
		app_sr_req      		=> '0'      						,	--	: in    std_logic;
		app_sr_active   		=> OPEN   							,	--	: out   std_logic;
		app_ref_req     		=> '0'     							,	--	: in    std_logic;
		app_ref_ack     		=> OPEN     						,	--	: out   std_logic;
		app_zq_req      		=> '0'      						,	--	: in    std_logic;
		app_zq_ack      		=> OPEN      						,	--	: out   std_logic;
		ui_clk          		=> MIG_CLK          				,	--	: out   std_logic;
		ui_clk_sync_rst 		=> OPEN 							,	--	: out   std_logic;
		init_calib_complete		=> OPEN								,	--	: out std_logic;
		-- System reset            
		sys_rst					=> XRST								,	--	: in    std_logic
		-- sys_clk_i BUFG OUT
	  	mmcm_clk_out   			=> CLK200MHz							--	: out std_logic
	);

	u_usr_fifo_wrt : usr_fifo_wrt
	generic map (
		UMWT_TRN_DT_BITS		=> UMWT_TRN_DT_BITS					,	--	: INTEGER := 128;
		MEM_AREA				=> MEM_AREA								--	: std_logic_vector(31 downto 0) := X"20000000"	-- 512MBYTE
	)                              
	port map (                      
--<< NbN / Zbg >>        
		RST						=> RST								,	--	: in	std_logic;						-- Zbg
		CLK						=> CLK200MHz						,	--	: in	std_logic;						-- NbN
--<< ADC >>                    
		ADC_DT_0				=> (others => '0')					,	--	: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_1				=> (others => '0')					,	--	: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_2				=> (others => '0')					,	--	: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_3				=> (others => '0')					,	--	: in	std_logic_vector(15 downto 0);	-- ADC DATA
--<< M >>                   
		user_gpio_out_00		=> user_gpio_out_00					,	--	: in	std_logic_vector(31 downto 0);
		user_gpio_in_00			=> user_gpio_in_00					,	--	: out	std_logic_vector(31 downto 0);
--<< FIFOM >>                   
		user_mwr_ad				=> user_mwr_ad						,	--	: out	std_logic_vector(31 downto 6);
		user_mwr_ad_en			=> user_mwr_ad_en					,	--	: out	std_logic;
		user_mwr_ad_afull		=> user_mwr_ad_afull				,	--	: in	std_logic;
		user_mwr_dt				=> user_mwr_dt						,	--	: out	std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
		user_mwr_dt_en			=> user_mwr_dt_en						--	: out	std_logic
	);

	u_usr_fifo_rd : usr_fifo_rd
	generic map (
		UMRD_ABITS_AD			=> UMRD_ABITS_AD					,	--	: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
		UMRD_TRN_DT_BITS		=> UMRD_TRN_DT_BITS					,	--	: INTEGER range 128 to 512 := 512;		-- 512p
		MEM_AREA				=> MEM_AREA								--	: std_logic_vector(31 downto 0) := X"20000000"	-- 512MBYTE
	)
	port map(
--<< NbN / Zbg >>
		RST						=> RST								,	--	: in	std_logic;						-- Zbg
		CLK						=> CLK200MHz						,	--	: in	std_logic;						-- NbN
--<< f[^o >>                 
		DT_OUT_0				=> OPEN								,	--	: out	std_logic_vector(15 downto 0);
		DT_OUT_1				=> OPEN								,	--	: out	std_logic_vector(15 downto 0);
		DT_OUT_2				=> OPEN								,	--	: out	std_logic_vector(15 downto 0);
		DT_OUT_3				=> OPEN								,	--	: out	std_logic_vector(15 downto 0);
--<< M >>                   
		user_gpio_out			=> user_gpio_out_01					,	--	: in	std_logic_vector(31 downto 0);
		user_gpio_in			=> user_gpio_in_01					,	--	: out	std_logic_vector(31 downto 0);
--<< FIFOM >>                   
		user_mrd_ad				=> user_mrd_ad						,	--	: out	std_logic_vector(31 downto 6);
		user_mrd_ad_en			=> user_mrd_ad_en					,	--	: out	std_logic;
		user_mrd_ad_afull		=> user_mrd_ad_afull				,	--	: in	std_logic;
		user_mrd_dt_en			=> user_mrd_dt_en					,	--	: out	std_logic;
		user_mrd_dt				=> user_mrd_dt						,	--	: in 	std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
		user_mrd_dt_count		=> user_mrd_dt_count					--	: in 	std_logic_vector(UMRD_ABITS_AD-1 downto 0)
	);

end RTL;
