----------------------------------------------------------------------
-- usr_fifo_rd
----------------------------------------------------------------------
-- Revision 00
-- Level	00
----------------------------------------------------------------------
-- File			: usr_fifo_rd.vhd
-- Entity		: usr_fifo_rd
-- Architecture	: RTL
-- Package		: gp
-- History -----------------------------------------------------------
-- Rev.Level	Date		Cause		Coded					Contents
-- 00.00		08/01/2011									new
----------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
library UNISIM;
use UNISIM.Vcomponents.ALL;

entity usr_fifo_rd is
	generic (
		UMRD_ABITS_AD			: INTEGER := 8;							-- AD-FIFO e(2**ABITS_AD)
		UMRD_TRN_DT_BITS		: INTEGER range 128 to 512 := 512;		-- 512p
		MEM_AREA				: std_logic_vector(31 downto 0) := X"20000000"	-- 512MBYTE
	);
	port (
--<< NbN / Zbg >>
		RST						: in	std_logic;						-- Zbg
		CLK						: in	std_logic;						-- NbN
--<< f[^o >>
		DT_OUT_0				: out	std_logic_vector(15 downto 0);
		DT_OUT_1				: out	std_logic_vector(15 downto 0);
		DT_OUT_2				: out	std_logic_vector(15 downto 0);
		DT_OUT_3				: out	std_logic_vector(15 downto 0);
--<< M >>
		user_gpio_out			: in	std_logic_vector(31 downto 0);
		user_gpio_in			: out	std_logic_vector(31 downto 0);
--<< FIFOM >>
		user_mrd_ad				: out	std_logic_vector(31 downto 6);
		user_mrd_ad_en			: out	std_logic;
		user_mrd_ad_afull		: in	std_logic;
		user_mrd_dt_en			: out	std_logic;
		user_mrd_dt				: in 	std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
		user_mrd_dt_count		: in 	std_logic_vector(UMRD_ABITS_AD-1 downto 0)
	);
end usr_fifo_rd;

architecture rtl of usr_fifo_rd is

--------------------------------------------------
-- signals
--------------------------------------------------
signal	dma_run 				: std_logic;
signal	dma_run_cont			: std_logic;
signal	dma_status_rst			: std_logic;
signal	dma_ch1_sel				: std_logic;
signal	dma_ch2_sel				: std_logic;
signal	dma_ch3_sel				: std_logic;							
signal	dma_ch4_sel				: std_logic;							
signal	dt_shift_cnt			: std_logic_vector(4 downto 0);
signal	dt_shift				: std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
signal	user_mrd_dt_swap		: std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
signal	dt_shift_ep				: std_logic;
signal	user_mrd_ad_org			: std_logic_vector(31 downto 0);
signal	user_mrd_ad_en0			: std_logic;
signal	fifo_read_req			: std_logic;
signal	user_mrd_dt_en_len		: std_logic_vector(31 downto 0);
signal	fifo_read_err			: std_logic;
signal	fifo_read_end_flg		: std_logic;
signal	inc_data_format_err		: std_logic;
signal	s0_user_mrd_dt_en		: std_logic;
signal	s1_user_mrd_dt_en		: std_logic;
signal	s0_user_mrd_dt			: std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
signal	s1_user_mrd_dt			: std_logic_vector(UMRD_TRN_DT_BITS-1 downto 0);
signal	user_mrd_dt_en0			: std_logic;
signal	user_mrd_ad0			: std_logic_vector(31 downto 6);

type	DEFINE_16_32_TYPE is array (0 to 15) of std_logic_vector(31 downto 0);
signal	s_user_gpio_out			: DEFINE_16_32_TYPE;

-- sequencer
type	s_st is (IDLE,HALF_IN,W_HALF_IN,OUTDT,WCLR);
signal	curr_a 				 	: s_st; 								-- Current State
signal	next_a					: s_st; 								-- Next State

begin

	----------------------------------------------------------
	-- 䃌WX^Ē`
	----------------------------------------------------------
	s_user_gpio_out(0)	<= user_gpio_out;
u_01_GEN : for i in 15 downto 1 generate
	process(CLK , RST) begin
		if(RST = '1') then
			s_user_gpio_out(i)	<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			s_user_gpio_out(i)	<= s_user_gpio_out(i-1);
		end if;
	end process;
end generate;
	dma_run 			<= s_user_gpio_out(7)(0);
	dma_run_cont		<= s_user_gpio_out(7)(1);
	dma_status_rst		<= s_user_gpio_out(2)(7);
	dma_ch1_sel			<= s_user_gpio_out(2)(08);
	dma_ch2_sel			<= s_user_gpio_out(2)(09);
	dma_ch3_sel			<= s_user_gpio_out(2)(10);
	dma_ch4_sel			<= s_user_gpio_out(2)(11);
	----------------------------------------------------------
	-- Xe[g}V
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			curr_a <= IDLE;	
		elsif(CLK'event and CLK = '1') then
			curr_a <= next_a;
		end if;
	end process;
	
	process(curr_a,dma_run,dma_run_cont,user_mrd_dt_count,fifo_read_req,fifo_read_end_flg) begin
		case curr_a is
			when IDLE =>												-- IDLE
				if(dma_run = '1' or dma_run_cont = '1')then 
					next_a <= HALF_IN;
				else
					next_a <= IDLE;
				end if;
			when HALF_IN | W_HALF_IN => 								-- FIFO~ς܂ő҂
				if(user_mrd_dt_count(UMRD_ABITS_AD-1) = '1')then
					next_a <= OUTDT;
				else
					next_a <= W_HALF_IN;
				end if;
			when OUTDT =>												-- DATA o͊Jn
				if(fifo_read_req = '0' and fifo_read_end_flg = '1')then
					next_a <= WCLR;
				else
					next_a <= OUTDT;
				end if;
			when WCLR =>												-- tONA҂
				if(dma_run = '1')then 
					next_a <= WCLR;
				else
					next_a <= IDLE;
				end if;
		end case;
	end process;
	----------------------------------------------------------
	-- f[^Vtg
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			dt_shift_cnt	<= (others => '0');
			dt_shift		<= (others => '0');
			dt_shift_ep		<= '0';
		elsif(CLK'event and CLK = '1') then
			if(dt_shift_cnt = 0)then
				dt_shift		<= user_mrd_dt_swap;
				dt_shift_ep		<= '1';
				if(dma_ch4_sel = '1')then
					dt_shift_cnt	<= "00111";
				elsif(dma_ch2_sel = '1')then  
					dt_shift_cnt	<= "01111";
				else                          
					dt_shift_cnt	<= "11111";
				end if;
			else
				dt_shift_ep		<= '0';
				dt_shift_cnt	<= dt_shift_cnt	- 1;
				if(dma_ch4_sel = '1')then
					dt_shift		<= X"0000000000000000" & dt_shift(UMRD_TRN_DT_BITS-1 downto 64);
				elsif(dma_ch2_sel = '1')then  
					dt_shift		<= X"00000000" & dt_shift(UMRD_TRN_DT_BITS-1 downto 32);
				else                          
					dt_shift		<= X"0000" & dt_shift(UMRD_TRN_DT_BITS-1 downto 16);
				end if;
			end if;
		end if;
	end process;
	u_swap_GEN : for i in UMRD_TRN_DT_BITS/32-1 downto 0 generate
		user_mrd_dt_swap(i*32+31 downto i*32+0) 	<= user_mrd_dt(i*32+7 downto i*32+0) & user_mrd_dt(i*32+15 downto i*32+8) & user_mrd_dt(i*32+23 downto i*32+16) & user_mrd_dt(i*32+31 downto i*32+24);
	end generate;
	----------------------------------------------------------
	-- f[^
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			DT_OUT_0		<= (others => '0');
			DT_OUT_1		<= (others => '0');
			DT_OUT_2		<= (others => '0');
			DT_OUT_3		<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			if(dma_ch4_sel = '1')then
				DT_OUT_0		<= dt_shift(16*00+15 downto 16*00+0);
				DT_OUT_1		<= dt_shift(16*01+15 downto 16*01+0);
				DT_OUT_2		<= dt_shift(16*02+15 downto 16*02+0);
				DT_OUT_3		<= dt_shift(16*03+15 downto 16*03+0);
			elsif(dma_ch2_sel = '1')then  
				DT_OUT_0		<= dt_shift(16*00+15 downto 16*00+0);
				DT_OUT_1		<= dt_shift(16*01+15 downto 16*01+0);
			else                          
				DT_OUT_0		<= dt_shift(16*00+15 downto 16*00+0);
			end if;
		end if;
	end process;
	----------------------------------------------------------
	--<< FIFOM >>
	----------------------------------------------------------
	user_mrd_ad_en	<= user_mrd_ad_en0;
	user_mrd_ad		<= user_mrd_ad0;
	process(CLK , RST) begin
		if(RST = '1') then
			user_mrd_ad0		<= (others => '0');
			user_mrd_ad_org		<= (others => '0');
			user_mrd_ad_en0		<= '0';
			fifo_read_req		<= '0';
		elsif(CLK'event and CLK = '1') then
			case curr_a is
				when HALF_IN => 
					fifo_read_req		<= '1';
					user_mrd_ad_org		<= (others => '0');
				when others => 
					if(dt_shift_ep = '1' and user_mrd_ad_afull = '0' and fifo_read_req = '1')then	-- o̓f[^[gP
						if(user_mrd_ad_org = MEM_AREA)then
							if(dma_run_cont = '1')then
								user_mrd_ad_org		<= X"00000040";
								user_mrd_ad0		<= (others => '0');
								user_mrd_ad_en0		<= '1';
							else
								fifo_read_req		<= '0';
							end if;
						else
							user_mrd_ad_org		<= user_mrd_ad_org + 64;
							user_mrd_ad0		<= user_mrd_ad_org(31 downto 6);
							user_mrd_ad_en0		<= '1';
						end if;
					else
						user_mrd_ad_en0		<= '0';
					end if;
			end case;
		end if;
	end process;
	user_mrd_dt_en		<= user_mrd_dt_en0;
	process(CLK , RST) begin
		if(RST = '1') then
			user_mrd_dt_en0		<= '0';
			user_mrd_dt_en_len	<= (others => '0');
			fifo_read_err		<= '0';
			fifo_read_end_flg	<= '0';
		elsif(CLK'event and CLK = '1') then
			case curr_a is
				when HALF_IN =>
					user_mrd_dt_en_len	<= MEM_AREA;
					fifo_read_end_flg	<= '0';
				when OUTDT =>			
					if(dt_shift_ep = '1' and fifo_read_end_flg = '0')then
						if(user_mrd_dt_en_len = 0)then
							fifo_read_end_flg	<= '1';
						else
							if(user_mrd_dt_count /= 0)then
								user_mrd_dt_en0		<= '1';
								user_mrd_dt_en_len	<= user_mrd_dt_en_len - 64;
							else
								fifo_read_err		<= '1';
							end if;
						end if;
					else
						user_mrd_dt_en0		<= '0';
						if(user_mrd_ad0 = 0 and user_mrd_ad_en0 = '1')then
							user_mrd_dt_en_len	<= user_mrd_dt_en_len + MEM_AREA;
						end if;
					end if;
				when others =>			
					if(dma_status_rst = '1')then
						fifo_read_err		<= '0';
					end if;
			end case;
		end if;
	end process;
	-- fobOp
	process(CLK , RST) begin
		if(RST = '1') then
			inc_data_format_err		<= '0';
			s0_user_mrd_dt_en		<= '0';
			s1_user_mrd_dt_en		<= '0';
			s0_user_mrd_dt			<= (others => '0');
			s1_user_mrd_dt			<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			s0_user_mrd_dt_en		<= user_mrd_dt_en0;
			s1_user_mrd_dt_en		<= s0_user_mrd_dt_en;
			case curr_a is
				when HALF_IN =>
					s0_user_mrd_dt			<= (others => '0');
					s1_user_mrd_dt			<= (others => '0');
				when OUTDT =>			
					if(s0_user_mrd_dt_en = '1')then
						s0_user_mrd_dt			<= user_mrd_dt;
						s1_user_mrd_dt			<= s0_user_mrd_dt;
					elsif(s1_user_mrd_dt_en = '1')then
						if(
							(s0_user_mrd_dt(32*00+31 downto 32*00+4) /= s0_user_mrd_dt(32*08+31 downto 32*08+4)) or 
							(s0_user_mrd_dt(32*01+31 downto 32*01+4) /= s0_user_mrd_dt(32*09+31 downto 32*09+4)) or 
							(s0_user_mrd_dt(32*02+31 downto 32*02+4) /= s0_user_mrd_dt(32*10+31 downto 32*10+4)) or 
							(s0_user_mrd_dt(32*03+31 downto 32*03+4) /= s0_user_mrd_dt(32*11+31 downto 32*11+4)) or 
							(s0_user_mrd_dt(32*04+31 downto 32*04+4) /= s0_user_mrd_dt(32*12+31 downto 32*12+4)) or 
							(s0_user_mrd_dt(32*05+31 downto 32*05+4) /= s0_user_mrd_dt(32*13+31 downto 32*13+4)) or 
							(s0_user_mrd_dt(32*06+31 downto 32*06+4) /= s0_user_mrd_dt(32*14+31 downto 32*14+4)) or 
							(s0_user_mrd_dt(32*07+31 downto 32*07+4) /= s0_user_mrd_dt(32*15+31 downto 32*15+4)) or 
								
							(s0_user_mrd_dt(32*00+3 downto 32*00+0) /= "0000") or 
							(s0_user_mrd_dt(32*01+3 downto 32*01+0) /= "0001") or 
							(s0_user_mrd_dt(32*02+3 downto 32*02+0) /= "0010") or 
							(s0_user_mrd_dt(32*03+3 downto 32*03+0) /= "0011") or 
							(s0_user_mrd_dt(32*04+3 downto 32*04+0) /= "0100") or 
							(s0_user_mrd_dt(32*05+3 downto 32*05+0) /= "0101") or 
							(s0_user_mrd_dt(32*06+3 downto 32*06+0) /= "0110") or 
							(s0_user_mrd_dt(32*07+3 downto 32*07+0) /= "0111") or 
							(s0_user_mrd_dt(32*08+3 downto 32*08+0) /= "1000") or 
							(s0_user_mrd_dt(32*09+3 downto 32*09+0) /= "1001") or 
							(s0_user_mrd_dt(32*10+3 downto 32*10+0) /= "1010") or 
							(s0_user_mrd_dt(32*11+3 downto 32*11+0) /= "1011") or 
							(s0_user_mrd_dt(32*12+3 downto 32*12+0) /= "1100") or 
							(s0_user_mrd_dt(32*13+3 downto 32*13+0) /= "1101") or 
							(s0_user_mrd_dt(32*14+3 downto 32*14+0) /= "1110") or 
							(s0_user_mrd_dt(32*15+3 downto 32*15+0) /= "1111"))then
							inc_data_format_err		<= '1';
						end if;
					end if;
				when others =>			
					if(dma_status_rst = '1')then
						inc_data_format_err		<= '0';
					end if;
			end case;
		end if;
	end process;
	----------------------------------------------------------
	-- DMA Xe[^X
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			user_gpio_in		<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			user_gpio_in(29 downto 6)	<= user_mrd_ad_org(29 downto 6);
			user_gpio_in(1)				<= fifo_read_err;
			user_gpio_in(4)				<= inc_data_format_err;
			if(curr_a = WCLR)then
				user_gpio_in(0)				<= '1';
			elsif(dma_status_rst = '1')then
				user_gpio_in(0)				<= '0';
			end if;
		end if;
	end process;

end rtl;
