----------------------------------------------------------------------
-- usr_fifo_wrt
----------------------------------------------------------------------
-- Revision 00
-- Level	00
----------------------------------------------------------------------
-- File			: usr_fifo_wrt.vhd
-- Entity		: usr_fifo_wrt
-- Architecture	: RTL
-- Package		: gp
-- History -----------------------------------------------------------
-- Rev.Level	Date		Cause		Coded					Contents
-- 00.00		08/01/2011									new
----------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
library UNISIM;
use UNISIM.Vcomponents.ALL;

entity usr_fifo_wrt is
	generic (
		UMWT_TRN_DT_BITS		: INTEGER range 64 to 512 := 128;
		MEM_AREA				: std_logic_vector(31 downto 0) := X"20000000"	-- 512MBYTE
	);
	port (
--<< NbN / Zbg >>
		RST						: in	std_logic;						-- Zbg
		CLK						: in	std_logic;						-- NbN
--<< ADC >>
		ADC_DT_0				: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_1				: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_2				: in	std_logic_vector(15 downto 0);	-- ADC DATA
		ADC_DT_3				: in	std_logic_vector(15 downto 0);	-- ADC DATA
--<< M >>
		user_gpio_out_00		: in	std_logic_vector(31 downto 0);
		user_gpio_in_00			: out	std_logic_vector(31 downto 0);
--<< FIFOM >>
		user_mwr_ad				: out	std_logic_vector(31 downto 6);
		user_mwr_ad_en			: out	std_logic;
		user_mwr_ad_afull		: in	std_logic;
		user_mwr_dt				: out	std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
		user_mwr_dt_en			: out	std_logic
	);
end usr_fifo_wrt;

architecture rtl of usr_fifo_wrt is

--------------------------------------------------
-- signals
--------------------------------------------------
signal	s0_user_gpio_out_00		: std_logic_vector(31 downto 0);
signal	s1_user_gpio_out_00		: std_logic_vector(31 downto 0);
signal	dma_run 				: std_logic;
signal	dma_run_cont			: std_logic;
signal	dma_status_rst			: std_logic;
signal	dma_ch1_sel				: std_logic;
signal	dma_ch2_sel				: std_logic;
signal	dma_ch3_sel				: std_logic;							
signal	dma_ch4_sel				: std_logic;							
signal	wrt_data				: std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
signal	wrt_data_sel			: std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
signal	wrt_data_sel_swap		: std_logic_vector(UMWT_TRN_DT_BITS-1 downto 0);
signal	wrt_data_en				: std_logic;
signal	wrt_data_en_cnt			: std_logic_vector(7 downto 0);
signal	fifo_dt_cnt				: std_logic_vector(31 downto 0);
signal	fifo_wrt_err			: std_logic;
signal	dma_end_p 				: std_logic;
signal	dma_test_mode			: std_logic;
signal	dma_test_ad_mode		: std_logic;
signal	adc_test				: std_logic_vector(15 downto 0);
signal	ADC_DT_0_sel			: std_logic_vector(15 downto 0);		-- ADC DATA
signal	ADC_DT_1_sel			: std_logic_vector(15 downto 0);		-- ADC DATA
signal	ADC_DT_2_sel			: std_logic_vector(15 downto 0);		-- ADC DATA
signal	ADC_DT_3_sel			: std_logic_vector(15 downto 0);		-- ADC DATA
signal	wrt_data_en_mask		: std_logic;
signal	wrt_data_en_mask_cnt	: std_logic_vector(5 downto 0);

type	DEFINE_16_32_TYPE is array (0 to 15) of std_logic_vector(31 downto 0);
signal	s_user_gpio_out_00		: DEFINE_16_32_TYPE;

-- sequencer
type	s_st is (IDLE,INIT,DMA,WCLR,S_END);
signal	curr_a 				 	: s_st; 								-- Current State
signal	next_a					: s_st; 								-- Next State

begin

	----------------------------------------------------------
	-- 䃌WX^Ē`
	----------------------------------------------------------
	s_user_gpio_out_00(0)	<= user_gpio_out_00;
u_01_GEN : for i in 15 downto 1 generate
	process(CLK , RST) begin
		if(RST = '1') then
			s_user_gpio_out_00(i)	<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			s_user_gpio_out_00(i)	<= s_user_gpio_out_00(i-1);
		end if;
	end process;
end generate;
	dma_run 			<= s_user_gpio_out_00(7)(0);
	dma_run_cont		<= s_user_gpio_out_00(7)(1);
	dma_status_rst		<= s_user_gpio_out_00(2)(7);
	dma_ch1_sel			<= s_user_gpio_out_00(2)(08);
	dma_ch2_sel			<= s_user_gpio_out_00(2)(09);
	dma_ch3_sel			<= s_user_gpio_out_00(2)(10);						
	dma_ch4_sel			<= s_user_gpio_out_00(2)(11);						
	dma_test_mode		<= s_user_gpio_out_00(2)(15);
	dma_test_ad_mode	<= s_user_gpio_out_00(2)(16);
	----------------------------------------------------------
	-- Xe[g}V
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			curr_a <= IDLE;	
		elsif(CLK'event and CLK = '1') then
			curr_a <= next_a;
		end if;
	end process;
	
	process(curr_a,dma_run,dma_run_cont,dma_end_p) begin
		case curr_a is
			when IDLE =>												-- IDLE
				if(dma_run = '1' or dma_run_cont = '1')then 
					next_a <= INIT;
				else
					next_a <= IDLE;
				end if;
			when INIT =>												-- DMA lZbg
				next_a <= DMA;
			when DMA =>													-- DMA START
				if(dma_end_p = '1')then
					if(dma_run_cont = '1')then 
						next_a <= INIT;
					else
						next_a <= WCLR;
					end if;
				else
					next_a <= DMA;
				end if;
			when WCLR =>												-- tONA҂
				if(dma_run = '1' or dma_run_cont = '1')then 
					next_a <= WCLR;
				else
					next_a <= S_END;
				end if;
			when S_END =>												-- 
				next_a <= IDLE;
		end case;
	end process;
	----------------------------------------------------------
	-- Cgf[^pbN
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			adc_test		<= (others => '0');
			ADC_DT_0_sel	<= (others => '0');							-- ADC DATA
			ADC_DT_1_sel	<= (others => '0');							-- ADC DATA
			ADC_DT_2_sel	<= (others => '0');							-- ADC DATA
			ADC_DT_3_sel	<= (others => '0');							-- ADC DATA
		elsif(CLK'event and CLK = '1') then
			adc_test		<= adc_test + 1;
			if(dma_test_mode = '0')then
				ADC_DT_0_sel	<= ADC_DT_0;
				ADC_DT_1_sel	<= ADC_DT_1;
				ADC_DT_2_sel	<= ADC_DT_2;
				ADC_DT_3_sel	<= ADC_DT_3;
			else
				ADC_DT_0_sel	<= "00" & adc_test(13 downto 0);
				ADC_DT_1_sel	<= "01" & adc_test(13 downto 0);
				ADC_DT_2_sel	<= "10" & adc_test(13 downto 0);
				ADC_DT_3_sel	<= "11" & adc_test(13 downto 0);
			end if;
		end if;
	end process;
	process(CLK , RST) begin
		if(RST = '1') then
			wrt_data		<= (others => '0');
			wrt_data_en		<= '0';
			wrt_data_en_cnt	<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			wrt_data_en_cnt	<= wrt_data_en_cnt + 1;
			if(dma_ch4_sel = '1')then
				case UMWT_TRN_DT_BITS is
					when 64 => 		wrt_data_en		<= '1';
									wrt_data		<= ADC_DT_3_sel & ADC_DT_2_sel & ADC_DT_1_sel & ADC_DT_0_sel;
					when 128 => 	if(wrt_data_en_cnt(0) = '0')then
										wrt_data_en		<= '1';
								 	else
										wrt_data_en		<= '0';
								 	end if;
									wrt_data		<= ADC_DT_3_sel & ADC_DT_2_sel & ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 64);
					when 256 => 	if(wrt_data_en_cnt(1 downto 0) = 0)then
										wrt_data_en		<= '1';
									else
										wrt_data_en		<= '0';
									end if;
									wrt_data		<= ADC_DT_3_sel & ADC_DT_2_sel & ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 64);
					when others =>	if(wrt_data_en_cnt(2 downto 0) = 0)then
										wrt_data_en		<= '1';
									else
										wrt_data_en		<= '0';
									end if;
									wrt_data		<= ADC_DT_3_sel & ADC_DT_2_sel & ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 64);
				end case;
			elsif(dma_ch2_sel = '1')then
				case UMWT_TRN_DT_BITS is
					when 64	 => 	if(wrt_data_en_cnt(0) = '0')then
										wrt_data_en		<= '1';
								 	else
										wrt_data_en		<= '0';
								 	end if;
									wrt_data		<= ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 32);
					when 128 => 	if(wrt_data_en_cnt(1 downto 0) = 0)then
										wrt_data_en		<= '1';
								 	else
										wrt_data_en		<= '0';
								 	end if;
									wrt_data		<= ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 32);
					when 256 => 	if(wrt_data_en_cnt(2 downto 0) = 0)then
										wrt_data_en		<= '1';
									else
										wrt_data_en		<= '0';
									end if;
									wrt_data		<= ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 32);
					when others =>	if(wrt_data_en_cnt(3 downto 0) = 0)then
										wrt_data_en		<= '1';
									else
										wrt_data_en		<= '0';
									end if;
									wrt_data		<= ADC_DT_1_sel & ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 32);
				end case;
			else
				case UMWT_TRN_DT_BITS is
					when 64	 => 	if(wrt_data_en_cnt(1 downto 0) = 0)then
										wrt_data_en		<= '1';
								 	else
										wrt_data_en		<= '0';
								 	end if;
									wrt_data		<= ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 16);
					when 128 => 	if(wrt_data_en_cnt(2 downto 0) = 0)then
										wrt_data_en		<= '1';
								 	else
										wrt_data_en		<= '0';
								 	end if;
									wrt_data		<= ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 16);
					when 256 => 	if(wrt_data_en_cnt(3 downto 0) = 0)then
										wrt_data_en		<= '1';
									else
										wrt_data_en		<= '0';
									end if;
									wrt_data		<= ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 16);
					when others =>	if(wrt_data_en_cnt(4 downto 0) = 0)then
										wrt_data_en		<= '1';
									else
										wrt_data_en		<= '0';
									end if;
									wrt_data		<= ADC_DT_0_sel & wrt_data(UMWT_TRN_DT_BITS-1 downto 16);
				end case;
			end if;
		end if;
	end process;
	----------------------------------------------------------
	-- wrt_data_en}XN
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			wrt_data_en_mask		<= '1';
			wrt_data_en_mask_cnt	<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			case curr_a is
				when IDLE =>						
					wrt_data_en_mask		<= '1';
					wrt_data_en_mask_cnt	<= (others => '0');
				when others =>
					wrt_data_en_mask_cnt	<= wrt_data_en_mask_cnt + 1;
					if(wrt_data_en_mask_cnt = 63)then
						wrt_data_en_mask		<= '0';
					end if;
			end case;
		end if;
	end process;
	----------------------------------------------------------
	--<< FIFOM >>
	----------------------------------------------------------
	u_swap_GEN : for i in UMWT_TRN_DT_BITS/32-1 downto 0 generate
		wrt_data_sel_swap(i*32+31 downto i*32+0) 	<= wrt_data_sel(i*32+7 downto i*32+0) & wrt_data_sel(i*32+15 downto i*32+8) & wrt_data_sel(i*32+23 downto i*32+16) & wrt_data_sel(i*32+31 downto i*32+24);
	end generate;
	process(CLK , RST) begin
		if(RST = '1') then
			user_mwr_ad			<= (others => '0');
			user_mwr_ad_en		<= '0';
			user_mwr_dt			<= (others => '0');
			user_mwr_dt_en		<= '0';
			fifo_dt_cnt			<= (others => '0');
			fifo_wrt_err		<= '0';
		elsif(CLK'event and CLK = '1') then
			if(dma_test_ad_mode	= '0')then
				user_mwr_dt			<= wrt_data_sel_swap;
			else
				user_mwr_dt			<= wrt_data_sel;
			end if;
			case curr_a is
				when INIT =>						
					fifo_dt_cnt			<= (others => '0');
				when DMA =>						
					if(wrt_data_en = '1' and wrt_data_en_mask = '0')then
--						user_mwr_dt			<= wrt_data_sel_swap;
						fifo_dt_cnt			<= fifo_dt_cnt + UMWT_TRN_DT_BITS/8;
						user_mwr_dt_en		<= not fifo_wrt_err;
						if((fifo_dt_cnt(5) = '1' and UMWT_TRN_DT_BITS = 256) or
						(fifo_dt_cnt(5 downto 4) = "11" and UMWT_TRN_DT_BITS = 128) or
						(fifo_dt_cnt(5 downto 3) = "111" and UMWT_TRN_DT_BITS = 64) or
						(UMWT_TRN_DT_BITS = 512))then
							user_mwr_ad			<= "00" & fifo_dt_cnt(29 downto 6);
							user_mwr_ad_en		<= not fifo_wrt_err;
							fifo_wrt_err		<= fifo_wrt_err or user_mwr_ad_afull;	-- G[o
						end if;
					else
						user_mwr_ad_en		<= '0';
						user_mwr_dt_en		<= '0';
					end if;
				when others => 
					if(dma_status_rst = '1')then
						fifo_wrt_err		<= '0';
					end if;
			end case;
		end if;
	end process;
	process(dma_test_ad_mode,fifo_dt_cnt,wrt_data) begin
		if(dma_test_ad_mode	= '0')then
			wrt_data_sel	<= wrt_data;
		else
			case UMWT_TRN_DT_BITS is
				when 64 =>	 	wrt_data_sel(32*00+31 downto 32*00+0)	<= ("00" & fifo_dt_cnt(31 downto 3) & '0');
							 	wrt_data_sel(32*01+31 downto 32*01+0)	<= ("00" & fifo_dt_cnt(31 downto 3) & '1');
				when 128 => 	wrt_data_sel(32*00+31 downto 32*00+0)	<= ("00" & fifo_dt_cnt(31 downto 4) & "00");
							 	wrt_data_sel(32*01+31 downto 32*01+0)	<= ("00" & fifo_dt_cnt(31 downto 4) & "01");
							 	wrt_data_sel(32*02+31 downto 32*02+0)	<= ("00" & fifo_dt_cnt(31 downto 4) & "10");
							 	wrt_data_sel(32*03+31 downto 32*03+0)	<= ("00" & fifo_dt_cnt(31 downto 4) & "11");
				when 256 => 	wrt_data_sel(32*00+31 downto 32*00+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "000");
							 	wrt_data_sel(32*01+31 downto 32*01+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "001");
							 	wrt_data_sel(32*02+31 downto 32*02+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "010");
							 	wrt_data_sel(32*03+31 downto 32*03+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "011");
							 	wrt_data_sel(32*04+31 downto 32*04+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "100");
							 	wrt_data_sel(32*05+31 downto 32*05+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "101");
							 	wrt_data_sel(32*06+31 downto 32*06+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "110");
							 	wrt_data_sel(32*07+31 downto 32*07+0)	<= ("00" & fifo_dt_cnt(31 downto 5) & "111");
				when others => 	wrt_data_sel(32*00+31 downto 32*00+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0000");
							 	wrt_data_sel(32*01+31 downto 32*01+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0001");
							 	wrt_data_sel(32*02+31 downto 32*02+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0010");
							 	wrt_data_sel(32*03+31 downto 32*03+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0011");
							 	wrt_data_sel(32*04+31 downto 32*04+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0100");
							 	wrt_data_sel(32*05+31 downto 32*05+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0101");
							 	wrt_data_sel(32*06+31 downto 32*06+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0110");
							 	wrt_data_sel(32*07+31 downto 32*07+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "0111");
							 	wrt_data_sel(32*08+31 downto 32*08+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1000");
							 	wrt_data_sel(32*09+31 downto 32*09+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1001");
							 	wrt_data_sel(32*10+31 downto 32*10+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1010");
							 	wrt_data_sel(32*11+31 downto 32*11+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1011");
							 	wrt_data_sel(32*12+31 downto 32*12+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1100");
							 	wrt_data_sel(32*13+31 downto 32*13+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1101");
							 	wrt_data_sel(32*14+31 downto 32*14+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1110");
							 	wrt_data_sel(32*15+31 downto 32*15+0)	<= ("00" & fifo_dt_cnt(31 downto 6) & "1111");
			end case;
		end if;
	end process;
	----------------------------------------------------------
	-- DMA pX
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			dma_end_p 			<= '0';
		elsif(CLK'event and CLK = '1') then
			case curr_a is
				when DMA =>						
					if(fifo_dt_cnt(29 downto 6) = MEM_AREA(29 downto 6))then
						dma_end_p 			<= '1';
					end if;
				when others =>
					dma_end_p 			<= '0';
			end case;
		end if;
	end process;
	----------------------------------------------------------
	-- DMA Xe[^X
	----------------------------------------------------------
	process(CLK , RST) begin
		if(RST = '1') then
			user_gpio_in_00		<= (others => '0');
		elsif(CLK'event and CLK = '1') then
			user_gpio_in_00(29 downto 6)	<= fifo_dt_cnt(29 downto 6);
			user_gpio_in_00(1)				<= fifo_wrt_err;
			if(curr_a = WCLR)then
				user_gpio_in_00(0)				<= '1';
			elsif(dma_status_rst = '1')then
				user_gpio_in_00(0)				<= '0';
			end if;
		end if;
	end process;

end rtl;
