/*******************************************************************************

	Project        : PCI express

	Component      : sample app

	Module         : dma.c

	Function       : dma sample

	Copyright (c) 2010 Skywave CO LTD.

-------------------------------------------------------------------------------
	[history]
	2010-09-27	Ver 1.00   M.Meguro    VK쐬

******************************************************************************/
//*******************************************/
//*    include								*/
//*******************************************/
#include	"PcieLib.h"

void
main(
	int		argc,
	char	*argv[]
)
{
	HANDLE	hDevice;
	int		ret;
	void	*phy,*usr;
	DWORD	value;
#ifdef	USE_INTR
	HANDLE	hEvent;
#endif

//	Pcie_SetDeviceAndEventName("ABCPcie","Global\\ABCIntrEvent");
	hDevice=Pcie_OpenDevice(0);
	if(hDevice!=INVALID_HANDLE_VALUE)
	{
		ret=Pcie_GetDMAMem(hDevice,8*1024*1024,&phy,&usr);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_GetDMAMem error=%d\n",ret);
			goto EXIT;
		}
		Pcie_WriteRegister32(hDevice,0,0x18,(DWORD)phy);
		Pcie_WriteRegister32(hDevice,0,0x1C,(DWORD)0x80000000);
		Pcie_WriteRegister32(hDevice,0,0x20,(DWORD)8*1024*1024);
		Pcie_WriteRegister32(hDevice,0,0x3C,(DWORD)0x80);
#ifdef	USE_INTR
		ret=Pcie_RegisterIntr(hDevice,0,0x00,0,0x00,0,0x04,0,0x11,&hEvent);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_RegisterIntr error=%d\n",ret);
			goto EXIT;
		}
		Pcie_WriteRegister32(hDevice,0,0x00,0x1001);	// start Read DMA with INTR
#else
		Pcie_WriteRegister32(hDevice,0,0x00,0x0001);	// start Read DMA
#endif

#ifdef	USE_INTR
		for(;;)
		{
			WaitForSingleObject(hEvent,INFINITE);
			ResetEvent(hEvent);
			Pcie_ReadRegister32(hDevice,0,0x04,&value);
			if(value & 0x11)
			{
				Pcie_WriteRegister32(hDevice,0,0x04,0x11);
				break;
			}
		}
		ret=Pcie_UnRegisterIntr(hDevice,hEvent);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_UnRegisterIntr error=%d\n",ret);
			goto EXIT;
		}
#else
		for(;;)
		{
			Pcie_ReadRegister32(hDevice,0,0x04,&value);
			if(value & 0x11)
			{
				Pcie_WriteRegister32(hDevice,0,0x04,0x11);
				break;
			}
		}
#endif
		ret=Pcie_RelDMAMem(hDevice);
		if(ret!=PCIE_NORMAL)
		{
			printf("Pcie_RelDMAMem error=%d\n",ret);
			goto EXIT;
		}
		Pcie_WriteRegister32(hDevice,0,0x00,0x00);
		printf("DMA OK\n");
	}
	else
		printf("driver open error\n");
EXIT:
	if(hDevice!=INVALID_HANDLE_VALUE)
		Pcie_CloseDevice(hDevice);
}
